package npg_qc::Schema::Result::RunTile;

# Created by DBIx::Class::Schema::Loader
# DO NOT MODIFY THE FIRST PART OF THIS FILE

use strict;
use warnings;

use base 'DBIx::Class::Core';


=head1 NAME

npg_qc::Schema::Result::RunTile

=cut

__PACKAGE__->table("run_tile");

=head1 ACCESSORS

=head2 id_run_tile

  data_type: 'bigint'
  extra: {unsigned => 1}
  is_auto_increment: 1
  is_nullable: 0

=head2 id_run

  data_type: 'bigint'
  extra: {unsigned => 1}
  is_nullable: 0

=head2 tile

  data_type: 'smallint'
  extra: {unsigned => 1}
  is_nullable: 0

=head2 position

  data_type: 'tinyint'
  extra: {unsigned => 1}
  is_nullable: 0

=head2 end

  data_type: 'char'
  is_nullable: 0
  size: 1

=head2 row

  data_type: 'smallint'
  extra: {unsigned => 1}
  is_nullable: 1

=head2 col

  data_type: 'tinyint'
  extra: {unsigned => 1}
  is_nullable: 1

=head2 avg_newz

  data_type: 'float'
  is_nullable: 1

=cut

__PACKAGE__->add_columns(
  "id_run_tile",
  {
    data_type => "bigint",
    extra => { unsigned => 1 },
    is_auto_increment => 1,
    is_nullable => 0,
  },
  "id_run",
  { data_type => "bigint", extra => { unsigned => 1 }, is_nullable => 0 },
  "tile",
  { data_type => "smallint", extra => { unsigned => 1 }, is_nullable => 0 },
  "position",
  { data_type => "tinyint", extra => { unsigned => 1 }, is_nullable => 0 },
  "end",
  { data_type => "char", is_nullable => 0, size => 1 },
  "row",
  { data_type => "smallint", extra => { unsigned => 1 }, is_nullable => 1 },
  "col",
  { data_type => "tinyint", extra => { unsigned => 1 }, is_nullable => 1 },
  "avg_newz",
  { data_type => "float", is_nullable => 1 },
);
__PACKAGE__->set_primary_key("id_run_tile");
__PACKAGE__->add_unique_constraint(
  "unq_idx_rt_idrun_end_position_tile",
  ["id_run", "end", "position", "tile"],
);

=head1 RELATIONS

=head2 cumulative_errors_by_cycle

Type: has_many

Related object: L<npg_qc::Schema::Result::CumulativeErrorsByCycle>

=cut

__PACKAGE__->has_many(
  "cumulative_errors_by_cycle",
  "npg_qc::Schema::Result::CumulativeErrorsByCycle",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 error_rate_reference_including_blanks

Type: has_many

Related object: L<npg_qc::Schema::Result::ErrorRateReferenceIncludingBlank>

=cut

__PACKAGE__->has_many(
  "error_rate_reference_including_blanks",
  "npg_qc::Schema::Result::ErrorRateReferenceIncludingBlank",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 error_rate_reference_no_blanks

Type: has_many

Related object: L<npg_qc::Schema::Result::ErrorRateReferenceNoBlank>

=cut

__PACKAGE__->has_many(
  "error_rate_reference_no_blanks",
  "npg_qc::Schema::Result::ErrorRateReferenceNoBlank",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 error_rate_relative_reference_cycle_nucleotides

Type: has_many

Related object: L<npg_qc::Schema::Result::ErrorRateRelativeReferenceCycleNucleotide>

=cut

__PACKAGE__->has_many(
  "error_rate_relative_reference_cycle_nucleotides",
  "npg_qc::Schema::Result::ErrorRateRelativeReferenceCycleNucleotide",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 error_rate_relative_sequence_bases

Type: has_many

Related object: L<npg_qc::Schema::Result::ErrorRateRelativeSequenceBase>

=cut

__PACKAGE__->has_many(
  "error_rate_relative_sequence_bases",
  "npg_qc::Schema::Result::ErrorRateRelativeSequenceBase",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 errors_by_cycle

Type: has_many

Related object: L<npg_qc::Schema::Result::ErrorsByCycle>

=cut

__PACKAGE__->has_many(
  "errors_by_cycle",
  "npg_qc::Schema::Result::ErrorsByCycle",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 errors_by_cycles_and_nucleotide

Type: has_many

Related object: L<npg_qc::Schema::Result::ErrorsByCycleAndNucleotide>

=cut

__PACKAGE__->has_many(
  "errors_by_cycles_and_nucleotide",
  "npg_qc::Schema::Result::ErrorsByCycleAndNucleotide",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 errors_by_nucleotide

Type: has_many

Related object: L<npg_qc::Schema::Result::ErrorsByNucleotide>

=cut

__PACKAGE__->has_many(
  "errors_by_nucleotide",
  "npg_qc::Schema::Result::ErrorsByNucleotide",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 information_contents_by_cycle

Type: has_many

Related object: L<npg_qc::Schema::Result::InformationContentByCycle>

=cut

__PACKAGE__->has_many(
  "information_contents_by_cycle",
  "npg_qc::Schema::Result::InformationContentByCycle",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 lane_qcs

Type: has_many

Related object: L<npg_qc::Schema::Result::LaneQc>

=cut

__PACKAGE__->has_many(
  "lane_qcs",
  "npg_qc::Schema::Result::LaneQc",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 log_likelihoods

Type: has_many

Related object: L<npg_qc::Schema::Result::LogLikelihood>

=cut

__PACKAGE__->has_many(
  "log_likelihoods",
  "npg_qc::Schema::Result::LogLikelihood",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 most_common_blank_patterns

Type: has_many

Related object: L<npg_qc::Schema::Result::MostCommonBlankPattern>

=cut

__PACKAGE__->has_many(
  "most_common_blank_patterns",
  "npg_qc::Schema::Result::MostCommonBlankPattern",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 most_common_words

Type: has_many

Related object: L<npg_qc::Schema::Result::MostCommonWord>

=cut

__PACKAGE__->has_many(
  "most_common_words",
  "npg_qc::Schema::Result::MostCommonWord",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 moves_z

Type: has_many

Related object: L<npg_qc::Schema::Result::MoveZ>

=cut

__PACKAGE__->has_many(
  "moves_z",
  "npg_qc::Schema::Result::MoveZ",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 tile_scores

Type: has_many

Related object: L<npg_qc::Schema::Result::TileScore>

=cut

__PACKAGE__->has_many(
  "tile_scores",
  "npg_qc::Schema::Result::TileScore",
  { "foreign.id_run_tile" => "self.id_run_tile" },
  { cascade_copy => 0, cascade_delete => 0 },
);


# Created by DBIx::Class::Schema::Loader v0.07000 @ 2011-06-30 15:20:49
# DO NOT MODIFY THIS OR ANYTHING ABOVE! md5sum:hFZuEi8GE01AeHjpLbaRDw


# You can replace this text with custom content, and it will be preserved on regeneration
1;
