#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2009-01-19
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: analysis_lane.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/api/analysis_lane.pm $
#
package npg_qc::api::analysis_lane;
use strict;
use warnings;
use Class::Std;
use Carp qw{cluck confess croak carp};
use English qw{-no_match_vars};
use base qw(npg_qc::api::base);

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };
{
  ## no critic (ProhibitUnusedVariables)
  my %id_analysis_lane_of :ATTR( :get<id_analysis_lane>, :set<id_analysis_lane> );
  my %id_analysis_of :ATTR( :get<id_analysis>, :set<id_analysis> );
  my %id_run_of :ATTR( :get<id_run>, :set<id_run> );
  my %position_of :ATTR( :get<position>, :set<position> );
  my %tile_count_of :ATTR( :get<tile_count>, :set<tile_count> );
  my %align_score_pf_of :ATTR( :get<align_score_pf>, :set<align_score_pf> );
  my %align_score_pf_err_of :ATTR( :get<align_score_pf_err>, :set<align_score_pf_err> );
  my %clusters_pf_of :ATTR( :get<clusters_pf>, :set<clusters_pf> );
  my %clusters_pf_err_of :ATTR( :get<clusters_pf_err>, :set<clusters_pf_err> );
  my %clusters_raw_of :ATTR( :get<clusters_raw>, :set<clusters_raw> );
  my %clusters_raw_err_of :ATTR( :get<clusters_raw_err>, :set<clusters_raw_err> );
  my %clusters_tilemean_raw_of :ATTR( :get<clusters_tilemean_raw>, :set<clusters_tilemean_raw> );
  my %cycle1_int_pf_of :ATTR( :get<cycle1_int_pf>, :set<cycle1_int_pf> );
  my %cycle1_int_pf_err_of :ATTR( :get<cycle1_int_pf_err>, :set<cycle1_int_pf_err> );
  my %cycle20_perc_int_of :ATTR( :get<cycle20_perc_int>, :set<cycle20_perc_int> );
  my %cycle20_perc_int_err_of :ATTR( :get<cycle20_perc_int_err>, :set<cycle20_perc_int_err> );
  my %cycle_10_20_av_perc_loss_pf_of :ATTR( :get<cycle_10_20_av_perc_loss_pf>, :set<cycle_10_20_av_perc_loss_pf> );
  my %cycle_10_20_av_perc_loss_pf_err_of :ATTR( :get<cycle_10_20_av_perc_loss_pf_err>, :set<cycle_10_20_av_perc_loss_pf_err> );
  my %cycle_2_10_av_perc_loss_pf_of :ATTR( :get<cycle_2_10_av_perc_loss_pf>, :set<cycle_2_10_av_perc_loss_pf> );
  my %cycle_2_10_av_perc_loss_pf_err_of :ATTR( :get<cycle_2_10_av_perc_loss_pf_err>, :set<cycle_2_10_av_perc_loss_pf_err> );
  my %cycle_2_4_av_int_pf_of :ATTR( :get<cycle_2_4_av_int_pf>, :set<cycle_2_4_av_int_pf> );
  my %cycle_2_4_av_int_pf_err_of :ATTR( :get<cycle_2_4_av_int_pf_err>, :set<cycle_2_4_av_int_pf_err> );
  my %equiv_perfect_clusters_pf_of :ATTR( :get<equiv_perfect_clusters_pf>, :set<equiv_perfect_clusters_pf> );
  my %equiv_perfect_clusters_raw_of :ATTR( :get<equiv_perfect_clusters_raw>, :set<equiv_perfect_clusters_raw> );
  my %error_rate_pf_of :ATTR( :get<error_rate_pf>, :set<error_rate_pf> );
  my %error_rate_pf_err_of :ATTR( :get<error_rate_pf_err>, :set<error_rate_pf_err> );
  my %lane_yield_of :ATTR( :get<lane_yield>, :set<lane_yield> );
  my %perc_align_pf_of :ATTR( :get<perc_align_pf>, :set<perc_align_pf> );
  my %perc_align_pf2_of :ATTR( :get<perc_align_pf2>, :set<perc_align_pf2> );
  my %perc_align_pf_err_of :ATTR( :get<perc_align_pf_err>, :set<perc_align_pf_err> );
  my %perc_clusters_pf_of :ATTR( :get<perc_clusters_pf>, :set<perc_clusters_pf> );
  my %perc_clusters_pf_err_of :ATTR( :get<perc_clusters_pf_err>, :set<perc_clusters_pf_err> );
  my %perc_error_rate_pf_of :ATTR( :get<perc_error_rate_pf>, :set<perc_error_rate_pf> );
  my %perc_error_rate_raw_of :ATTR( :get<perc_error_rate_raw>, :set<perc_error_rate_raw> );
  my %perc_phasing_of :ATTR( :get<perc_phasing>, :set<perc_phasing> );
  my %perc_prephasing_of :ATTR( :get<perc_prephasing>, :set<perc_prephasing> );
  my %perc_retained_of :ATTR( :get<perc_retained>, :set<perc_retained> );
  ## use critic

  sub fields {
    return (qw(
      id_analysis_lane
      id_analysis
      id_run
      position
      tile_count
      align_score_pf
      align_score_pf_err
      clusters_pf
      clusters_pf_err
      clusters_raw
      clusters_raw_err
      clusters_tilemean_raw
      cycle1_int_pf
      cycle1_int_pf_err
      cycle20_perc_int
      cycle20_perc_int_err
      cycle_10_20_av_perc_loss_pf
      cycle_10_20_av_perc_loss_pf_err
      cycle_2_10_av_perc_loss_pf
      cycle_2_10_av_perc_loss_pf_err
      cycle_2_4_av_int_pf
      cycle_2_4_av_int_pf_err
      equiv_perfect_clusters_pf
      equiv_perfect_clusters_raw
      error_rate_pf
      error_rate_pf_err
      lane_yield
      perc_align_pf
      perc_align_pf2
      perc_align_pf_err
      perc_clusters_pf
      perc_clusters_pf_err
      perc_error_rate_pf
      perc_error_rate_raw
      perc_phasing
      perc_prephasing
      perc_retained
    ));
  };
  sub id_analysis_lane {
    my ($self, $id_analysis_lane) = @_;
    return $self->store_and_return(q{id_analysis_lane}, $id_analysis_lane);
  }
  sub id_analysis {
    my ($self, $id_analysis) = @_;
    return $self->store_and_return(q{id_analysis}, $id_analysis);
  }
  sub id_run {
    my ($self, $id_run) = @_;
    return $self->store_and_return(q{id_run}, $id_run);
  }
  sub position {
    my ($self, $position) = @_;
    return $self->store_and_return(q{position}, $position);
  }
  sub tile_count {
    my ($self, $tile_count) = @_;
    return $self->store_and_return(q{tile_count}, $tile_count);
  }
  sub align_score_pf {
    my ($self, $align_score_pf) = @_;
    return $self->store_and_return(q{align_score_pf}, $align_score_pf);
  }
  sub align_score_pf_err {
    my ($self, $align_score_pf_err) = @_;
    return $self->store_and_return(q{align_score_pf_err}, $align_score_pf_err);
  }
  sub clusters_pf {
    my ($self, $clusters_pf) = @_;
    return $self->store_and_return(q{clusters_pf}, $clusters_pf);
  }
  sub clusters_pf_err {
    my ($self, $clusters_pf_err) = @_;
    return $self->store_and_return(q{clusters_pf_err}, $clusters_pf_err);
  }
  sub clusters_raw {
    my ($self, $clusters_raw) = @_;
    return $self->store_and_return(q{clusters_raw}, $clusters_raw);
  }
  sub clusters_raw_err {
    my ($self, $clusters_raw_err) = @_;
    return $self->store_and_return(q{clusters_raw_err}, $clusters_raw_err);
  }
  sub clusters_tilemean_raw {
    my ($self, $clusters_tilemean_raw) = @_;
    return $self->store_and_return(q{clusters_tilemean_raw}, $clusters_tilemean_raw);
  }
  sub cycle1_int_pf {
    my ($self, $cycle1_int_pf) = @_;
    return $self->store_and_return(q{cycle1_int_pf}, $cycle1_int_pf);
  }
  sub cycle1_int_pf_err {
    my ($self, $cycle1_int_pf_err) = @_;
    return $self->store_and_return(q{cycle1_int_pf_err}, $cycle1_int_pf_err);
  }
  sub cycle20_perc_int {
    my ($self, $cycle20_perc_int) = @_;
    return $self->store_and_return(q{cycle20_perc_int}, $cycle20_perc_int);
  }
  sub cycle20_perc_int_err {
    my ($self, $cycle20_perc_int_err) = @_;
    return $self->store_and_return(q{cycle20_perc_int_err}, $cycle20_perc_int_err);
  }
  sub cycle_10_20_av_perc_loss_pf {
    my ($self, $cycle_10_20_av_perc_loss_pf) = @_;
    return $self->store_and_return(q{cycle_10_20_av_perc_loss_pf}, $cycle_10_20_av_perc_loss_pf);
  }
  sub cycle_10_20_av_perc_loss_pf_err {
    my ($self, $cycle_10_20_av_perc_loss_pf_err) = @_;
    return $self->store_and_return(q{cycle_10_20_av_perc_loss_pf_err}, $cycle_10_20_av_perc_loss_pf_err);
  }
  sub cycle_2_10_av_perc_loss_pf {
    my ($self, $cycle_2_10_av_perc_loss_pf) = @_;
    return $self->store_and_return(q{cycle_2_10_av_perc_loss_pf}, $cycle_2_10_av_perc_loss_pf);
  }
  sub cycle_2_10_av_perc_loss_pf_err {
    my ($self, $cycle_2_10_av_perc_loss_pf_err) = @_;
    return $self->store_and_return(q{cycle_2_10_av_perc_loss_pf_err}, $cycle_2_10_av_perc_loss_pf_err);
  }
  sub cycle_2_4_av_int_pf {
    my ($self, $cycle_2_4_av_int_pf) = @_;
    return $self->store_and_return(q{cycle_2_4_av_int_pf}, $cycle_2_4_av_int_pf);
  }
  sub cycle_2_4_av_int_pf_err {
    my ($self, $cycle_2_4_av_int_pf_err) = @_;
    return $self->store_and_return(q{cycle_2_4_av_int_pf_err}, $cycle_2_4_av_int_pf_err);
  }
  sub equiv_perfect_clusters_pf {
    my ($self, $equiv_perfect_clusters_pf) = @_;
    return $self->store_and_return(q{equiv_perfect_clusters_pf}, $equiv_perfect_clusters_pf);
  }
  sub equiv_perfect_clusters_raw {
    my ($self, $equiv_perfect_clusters_raw) = @_;
    return $self->store_and_return(q{equiv_perfect_clusters_raw}, $equiv_perfect_clusters_raw);
  }
  sub error_rate_pf {
    my ($self, $error_rate_pf) = @_;
    return $self->store_and_return(q{error_rate_pf}, $error_rate_pf);
  }
  sub error_rate_pf_err {
    my ($self, $error_rate_pf_err) = @_;
    return $self->store_and_return(q{error_rate_pf_err}, $error_rate_pf_err);
  }
  sub lane_yield {
    my ($self, $lane_yield) = @_;
    return $self->store_and_return(q{lane_yield}, $lane_yield);
  }
  sub perc_align_pf {
    my ($self, $perc_align_pf) = @_;
    return $self->store_and_return(q{perc_align_pf}, $perc_align_pf);
  }
  sub perc_align_pf2 {
    my ($self, $perc_align_pf2) = @_;
    return $self->store_and_return(q{perc_align_pf2}, $perc_align_pf2);
  }
  sub perc_align_pf_err {
    my ($self, $perc_align_pf_err) = @_;
    return $self->store_and_return(q{perc_align_pf_err}, $perc_align_pf_err);
  }
  sub perc_clusters_pf {
    my ($self, $perc_clusters_pf) = @_;
    return $self->store_and_return(q{perc_clusters_pf}, $perc_clusters_pf);
  }
  sub perc_clusters_pf_err {
    my ($self, $perc_clusters_pf_err) = @_;
    return $self->store_and_return(q{perc_clusters_pf_err}, $perc_clusters_pf_err);
  }
  sub perc_error_rate_pf {
    my ($self, $perc_error_rate_pf) = @_;
    return $self->store_and_return(q{perc_error_rate_pf}, $perc_error_rate_pf);
  }
  sub perc_error_rate_raw {
    my ($self, $perc_error_rate_raw) = @_;
    return $self->store_and_return(q{perc_error_rate_raw}, $perc_error_rate_raw);
  }
  sub perc_phasing {
    my ($self, $perc_phasing) = @_;
    return $self->store_and_return(q{perc_phasing}, $perc_phasing);
  }
  sub perc_prephasing {
    my ($self, $perc_prephasing) = @_;
    return $self->store_and_return(q{perc_prephasing}, $perc_prephasing);
  }
  sub perc_retained {
    my ($self, $perc_retained) = @_;
    return $self->store_and_return(q{perc_retained}, $perc_retained);
  }
  sub store_and_return {
    my ($self, $method, $value) = @_;
    if ($value) {
      my $set_method = 'set_'.$method;
      $self->$set_method($value);
    }
    my $get_method = 'get_'.$method;
    return $self->$get_method();
  }
}
1;
__END__

=head1 NAME

npg_qc::api::analysis_lane

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

  my $oAnalysisLane = npg_qc::api::analysis_lane->new({
    util => $oNPG_QC_Util,
  });

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fields - list of accessors for this object

=head2 id_analysis_lane - accessor for id_analysis_lane

=head2 id_analysis - accessor for id_analysis

=head2 id_run - accessor for id_run

=head2 position - accessor for position

=head2 tile_count - accessor for tile_count

=head2 align_score_pf - accessor for align_score_pf

=head2 align_score_pf_err - accessor for align_score_pf_err

=head2 clusters_pf - accessor for clusters_pf

=head2 clusters_pf_err - accessor for clusters_pf_err

=head2 clusters_raw - accessor for clusters_raw

=head2 clusters_raw_err - accessor for clusters_raw_err

=head2 clusters_tilemean_raw - accessor for clusters_tilemean_raw

=head2 cycle1_int_pf - accessor for cycle1_int_pf

=head2 cycle1_int_pf_err - accessor for cycle1_int_pf_err

=head2 cycle20_perc_int - accessor for cycle20_perc_int

=head2 cycle20_perc_int_err - accessor for cycle20_perc_int_err

=head2 cycle_10_20_av_perc_loss_pf - accessor for cycle_10_20_av_perc_loss_pf

=head2 cycle_10_20_av_perc_loss_pf_e - accessor for cycle_10_20_av_perc_loss_pf_e

=head2 cycle_2_10_av_perc_loss_pf - accessor for cycle_2_10_av_perc_loss_pf

=head2 cycle_2_10_av_perc_loss_pf_er - accessor for cycle_2_10_av_perc_loss_pf_er

=head2 cycle_2_4_av_int_pf - accessor for cycle_2_4_av_int_pf

=head2 cycle_2_4_av_int_pf_err - accessor for cycle_2_4_av_int_pf_err

=head2 equiv_perfect_clusters_pf - accessor for equiv_perfect_clusters_pf

=head2 equiv_perfect_clusters_raw - accessor for equiv_perfect_clusters_raw

=head2 error_rate_pf - accessor for error_rate_pf

=head2 error_rate_pf_err - accessor for error_rate_pf_err

=head2 lane_yield - accessor for lane_yield

=head2 perc_align_pf - accessor for perc_align_pf

=head2 perc_align_pf2 - accessor for perc_align_pf2

=head2 perc_align_pf_err - accessor for perc_align_pf_err

=head2 perc_clusters_pf - accessor for perc_clusters_pf

=head2 perc_clusters_pf_err - accessor for perc_clusters_pf_err

=head2 perc_error_rate_pf - accessor for perc_error_rate_pf

=head2 perc_error_rate_raw - accessor for perc_error_rate_raw

=head2 perc_phasing - accessor for perc_phasing

=head2 perc_prephasing - accessor for perc_prephasing

=head2 perc_retained - accessor for perc_retained

=head2 store_and_retrieve - refactored code for all accessors to use

=head2 cycle_10_20_av_perc_loss_pf_err
=head2 cycle_2_10_av_perc_loss_pf_err
=head2 store_and_return


=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
English
Class::Std
Carp
npg::api:base

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
