#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-02-19
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: Recipe.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/api/loader/Recipe.pm $
#
package npg_qc::api::loader::Recipe;
use strict;
use warnings;
use Moose;
use Carp qw{carp cluck croak confess};
use English qw{-no_match_vars};
use Readonly;
use File::Basename;
use Perl6::Slurp;
use Digest::MD5 qw(md5 md5_hex);

extends qw{npg_qc::api::loader::base};

Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };

## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_qc::api::loader::Recipe

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

=head1 DESCRIPTION

This module read the recipe xml file and tile layout txt fiel under Config directory
to populate two tables run_recipe an reciep_file

=head1 SUBROUTINES/METHODS

=head2 file_name

run recipe file name
 
=cut
has 'file_name'    => (isa           => q{Str},
                       is            => q{rw},
                       lazy_build    => 1,
                       documentation => q{run recipe file name},
                      );

sub _build_file_name {

  my $self = shift;
  my @files = glob $self->runfolder_path.q{/Recipe*.xml};
  if(scalar @files != 1){
    $self->log( q{No Recipe file or there are more than one Recipe xml available:}.$self->runfolder_path );
    return q{};
  }

  return $files[0];
}

=head2 run

=cut

sub run {
  my ($self) = @_;

  $self->log(q{Loading run recipe file for Run } . $self->id_run() . q{ into QC database});

  $self->SUPER::run();

  $self->_read_run_recipe();

  $self->schema->txn_commit();

  return 1;
}


sub _read_run_recipe {
    my ($self) = @_;

    my $cycle_count = $self->cycle_count();
    my $tile_count = $self->tile_count();
    my $lane_count = $self->lane_count();

    my $col_count = 0;
    eval{
        $col_count= $self->tilelayout_columns();
        1;
    } or do {
        $self->log($EVAL_ERROR);
    };

    my $cycle_read1;
    my @read1_cycle_range = $self->read1_cycle_range();
    if(scalar @read1_cycle_range == 2){
       $cycle_read1 = $read1_cycle_range[1] - $read1_cycle_range[0] + 1;
    }

    my $cycle_read2;
    my @read2_cycle_range = $self->read2_cycle_range();
    if(scalar @read2_cycle_range == 2){
       $cycle_read2 = $read2_cycle_range[1] - $read2_cycle_range[0] + 1;
    }

    my $first_indexing_cycle_number;
    my $last_indexing_cycle_number;

    my @read_index_cycle_range = $self->indexing_cycle_range();
    if(scalar @read_index_cycle_range == 2){
       $first_indexing_cycle_number = $read_index_cycle_range[0];
       $last_indexing_cycle_number  = $read_index_cycle_range[1];
    }

    my $id_recipe_file;

    if(-e $self->file_name()){

        my $xml = slurp $self->file_name();
	my $md5 = md5_hex($xml);

        my $recipe_file_fields = {
               file_name   => basename($self->file_name()),
               md5         => $md5,
               xml         => $xml,
           };

        my $recipe_file_obj = $self->schema->resultset('RecipeFile')->update_or_create( $recipe_file_fields );
	$id_recipe_file = $recipe_file_obj->id_recipe_file();
    }

    my $run_recipe_fields = {
               id_run               => $self->id_run(),
               cycle                => $cycle_count,
               lane                 => $lane_count,
               tile                 => $tile_count,
               col                  => $col_count,
               first_indexing_cycle => $first_indexing_cycle_number,
               last_indexing_cycle  => $last_indexing_cycle_number,
               cycle_read1          => $cycle_read1,
               cycle_read2          => $cycle_read2,
               id_recipe_file       => $id_recipe_file,
             };

    $self->schema->resultset('RunRecipe')->update_or_create( $run_recipe_fields );

    return 1;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item MooseX::Getopt

=item npg_qc::api::loader::base

=item srpipe::identify::missing::tiles

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
