#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-02-19
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: Run_Config.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/api/loader/Run_Config.pm $
#
package npg_qc::api::loader::Run_Config;
use strict;
use warnings;
use Moose;
use Carp qw{carp cluck croak confess};
use English qw{-no_match_vars};
use Readonly;
use Perl6::Slurp;

extends qw{npg_qc::api::loader::base};

Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $VALUE_FOR_DATA_MINING_EFFICIENCY => -1;

## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_qc::api::loader::Run_Config

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 file_name

config file name
 
=cut
has 'file_name'    => (isa           => q{Str},
                       is            => q{rw},
                       lazy_build    => 1,
                       documentation => q{file name for run config text file},
                      );

sub _build_file_name {
  my $self = shift;
  return $self->recalibrated_path().q{/config.txt};
}

=head2 run

=cut

sub run {
  my ($self) = @_;

  $self->log(q{Loading Illumina run config file for Run } . $self->id_run() . q{ into QC database});

  my $config_file = $self->file_name();
  if(! -e $config_file){
    $self->log("Config file not exists!\n$config_file");
    return 1;
  }

  my $file_content = slurp $config_file;

  $self->schema->resultset('RunConfig')->update_or_create(
                                         {
                                          id_run      => $self->id_run(),
                                          config_text => $file_content,
                                         }
                                       );

  $self->schema->txn_commit();

  return 1;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item MooseX::Getopt

=item npg_qc::api::loader::base

=item Perl6::Slurp

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
