#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-02-19
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: Runlog.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/api/loader/Runlog.pm $
#
package npg_qc::api::loader::Runlog;
use strict;
use warnings;
use Moose;
use Carp qw{carp cluck croak confess};
use English qw{-no_match_vars};
use Readonly;
use npg_qc::api::run_caching;

extends qw{npg_qc::api::loader::base};

Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };

## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_qc::api::loader::Runlog

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 file_name

runlog file name
 
=cut
has 'file_name'    => (isa           => q{Str},
                       is            => q{rw},
                       lazy_build    => 1,
                       documentation => q{runlog file name},
                      );

sub _build_file_name {
  my $self = shift;

  return $self->runfolder_path().q{/RunLog_concat.xml}
}

has '_current_lane'        => (isa       => q{Int},
                               is        => q{rw},
                               );
has '_current_tile'        => (isa       => q{Int},
                               is        => q{rw},
                               );
has '_current_cycle_num'   => (isa       => q{Int},
                               is        => q{rw},
                              );
has '_movez_list'          => (isa       => q{HashRef},
                              is         => q{rw},
                              );
has '_tile_layout'         => (isa       => q{HashRef},
                               is        => q{rw},
                              );

=head2 run

=cut

sub run {
  my ($self) = @_;

  $self->log(q{Loading runlog files for Run } . $self->id_run() . q{ into QC database});

  $self->SUPER::run();

  my $runlog_file_name = $self->file_name();

  if(! -e $runlog_file_name){
    $self->log("There is no runlog file available\n$runlog_file_name");
    return;
  }

  $self->_extract_movez($runlog_file_name);
  $self->schema->txn_commit();

  #caching number of movez_tiles out of range        
  my $api_run_caching = npg_qc::api::run_caching->new({util => $self->util});
  $self->log(q{Caching number of movez tiles out of range});
  $api_run_caching->cache_query_movez_tiles($self->id_run(), 1, 1);
  if ($self->is_paired_read()) {
      $api_run_caching->cache_query_movez_tiles($self->id_run(), 2, 1);
  }
  return 1;
}

sub _extract_movez{
    my ($self, $runlog_file_url) = @_;

    open my $logfile, '<', $runlog_file_url or croak "can't open $runlog_file_url\n";
    $self->_read_log($logfile);
    close $logfile or croak "can't close the file handler $runlog_file_url\n";

    return 1;
}

sub _read_log{
    my ($self, $logfile)= @_;

    $self->log(q{Reading runlog xml});

    $self->_movez_list({});
    $self->_tile_layout({});

    my $line_count =0;

    my $start_cycle = 0;
    my $end_cycle = 0;

    my $cycle_tag_open = 0;
    my $cycle_tag_close = 1;

    my $tile_tag_open = 0;
    my $tile_tag_close = 1;


    while (my $line = <$logfile>) {
      $line_count++;

      #check open cycle or close cycle
      $start_cycle = $self->_start_tag($line, 'IMAGE_CYCLE');
      $end_cycle = $self->_end_tag($line, 'IMAGE_CYCLE');
      if($start_cycle && $end_cycle){
        #carp "ignore one line IMAGE_CYCLE tag at line $line_count: $line";
      }elsif ($start_cycle){
        $tile_tag_open = 0;
        $tile_tag_close = 1;

        if(!$cycle_tag_close) {
         # carp "cycle tag not close before line $line_count: $line";
          $cycle_tag_close = 1;
        }

        my ($current_cycle_num) =  $line =~ /<IMAGE_CYCLE.+Cycle="(\d+)"/mxs;
        if(defined $current_cycle_num){
          $self->_current_cycle_num($current_cycle_num);
        }#else{
          #carp "cycle number missing at line $line_count: $line";
        #}

        #print $current_cycle_num, ":$line";
        $cycle_tag_open = 1;
        $cycle_tag_close = 0;

      }elsif ($end_cycle){
        if(!$cycle_tag_open){
          #carp "cycle tag not open before line $line_count: $line";
        }
       # print $current_cycle_num, "-$line";
        $cycle_tag_close = 1;
        $cycle_tag_open = 0;
      }

      #if within one cycle
      if($cycle_tag_open && !$cycle_tag_close){

       #check open tile or close tile
        my $start_tile_tag = $self->_start_tag($line, 'TILE');
        my $end_tile_tag = $self->_end_tag($line, 'TILE');

        if($start_tile_tag && !$end_tile_tag){

          if(!$tile_tag_close){
            #carp "title tag not close before line $line_count: $line";
            $tile_tag_close = 1;
          }
          $self->_current_lane(0);
          $self->_current_tile(0);
          $tile_tag_open = 1;
          $tile_tag_close = 0;

        }elsif($end_tile_tag && !$start_tile_tag){
          $tile_tag_close = 1;
          $tile_tag_open = 0;
          $self->_current_lane(0);
          $self->_current_tile(0);
        }

        if($tile_tag_open && !$tile_tag_close){
           $self->_parse_movez_line($line, $line_count);
           $self->_parse_tile_line($line);
        }
      }
    }

    $self->_save_data();
    return;
}

sub _save_data{
    my ($self) = @_;

    $self->log(q{Saving data into database});

    foreach my $lane (sort keys %{$self->_movez_list()} ) {

      $self->log (qq{Lane: $lane});
      my $tile_list_per_lane = $self->_movez_list()->{$lane};

      foreach my $tile (sort keys %{$tile_list_per_lane}){

         my $cycle_list_per_tile = $tile_list_per_lane->{$tile};

         my $id_run_tile = $self->get_id_run_tile($lane, 1, $tile);

         my $sum_newz = 0;
         my $num_newz = 0;
         foreach my $cycle (sort{ $a <=> $b } keys %{$cycle_list_per_tile }){

           my $move_z = $cycle_list_per_tile->{$cycle};
           $move_z->{id_run_tile} = $id_run_tile;
           $self->schema->resultset('MoveZ')
                      ->update_or_create( $move_z );

           if($cycle != 1){
             $sum_newz += $move_z->{newz};
             $num_newz ++;
           }
         }
         my $avg_newz;

         if($num_newz > 0){
            $avg_newz = $sum_newz / $num_newz;
         }

         $self->schema->resultset('RunTile')
                      ->update_or_create({
                         id_run_tile => $id_run_tile,
                         row         => $self->_tile_layout()->{$lane}->{$tile}->{row},
                         col         => $self->_tile_layout()->{$lane}->{$tile}->{col},
                         avg_newz    => $avg_newz,
                      });
       }
    }
    return;
}

sub _parse_tile_line{
    my ($self, $line) = @_;

    if($line =~ /<TILE/mxs){
             #<TILE start="307999047" Lane="0" Column="0" Row="16" TileIndex="16" NumImages="96 (wait)" ImageQueue="0">
             my ($current_lane) = $line =~ /Lane="(\d+)"/smxi;
             my ($current_col) = $line =~ /Column="(\d+)"/smxi;
             my ($current_row) = $line =~ /Row="(\d+)"/smxi;
             my ($current_tile) = $line =~ /TileIndex="(\d+)"/smxi;

             if(defined $current_lane && defined $current_col && defined $current_row && defined $current_tile){
               #increase 1, to make they count from 1
               my $current_lane_from1 = $current_lane + 1;
               my $current_tile_from1 = $current_tile + 1;
               $self->_current_lane($current_lane_from1);
               $self->_current_tile($current_tile_from1);
               $self->_tile_layout()->{$current_lane_from1}->{$current_tile_from1}
                                             = {row=>$current_row, col =>$current_col};
             }
             #else{
               #carp "missing data at line: $line";
             #}

    }
    return;
}

sub _parse_movez_line{
    my ($self, $line) = @_;

    if($line =~ /<MOVE_Z/mxs){

      # <MOVE_Z start="307974797" stop="307974797" CurrentZ="16970" TargetZ="16800" NewZ="16970" MoveZ="N"
      my ($start) = $line =~ /start="(\-?\d+)"/smxi;
      my ($stop) = $line =~ /stop="(\-?\d+)"/smxi;
      my ($currentz) = $line =~ /CurrentZ="(\-?\d+)"/smxi;
      my ($targetz) = $line =~ /TargetZ="(\-?\d+)"/smxi;
      my ($newz) = $line =~ /NewZ="(\-?\d+)"/smxi;
      my ($move) = $line =~ /MoveZ="(\w)"/smxi;
      if(  defined $start
        && defined $stop
        && defined $currentz
        && defined $targetz
        && defined $newz
        && defined $move ){

        my $move_value = 0;
        if($move eq 'Y'){
        	     $move_value = 1;
        }
        my $current_move_z = {};

        $current_move_z->{start} = $start;
        $current_move_z->{stop}  = $stop;
        $current_move_z->{currentz} = $currentz;
        $current_move_z->{targetz}  = $targetz;
        $current_move_z->{newz}     = $newz;
        $current_move_z->{move}     = $move_value;
        $current_move_z->{cycle}    = $self->_current_cycle_num();

        my $current_lane = $self->_current_lane();
        my $current_tile = $self->_current_tile();
        my $current_cycle = $self->_current_cycle_num();

        if( $current_tile !=0  && $current_lane != 0){
          my $old_move_z = $self->_movez_list()->{$current_lane}->{$current_tile}->{$current_cycle};
          if (($old_move_z && $old_move_z->{start}< $current_move_z->{start}) || !$old_move_z){
    	        $self->_movez_list()->{$current_lane}->{$current_tile}->{$current_cycle} = $current_move_z;

          }
        }
      }
      #else{
         #carp "data missing in line: $line";
      #}

    }
    return;
}

sub _start_tag{
    my ($self, $line, $tag_name) = @_;

    return $line =~ /<$tag_name/mxs;
}

sub _end_tag{
    my ($self, $line, $tag_name) = @_;

    return $line =~ /<\/$tag_name>/mxs || $line =~ /<$tag_name.+\/>/msx;
}


no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item MooseX::Getopt

=item npg_qc::api::loader::base

=item Statistics::Lite

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
