#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-01-27
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: Role.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/api/monitoring/Role.pm $
#

package npg_qc::api::monitoring::Role;

use Moose::Role;
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use npg_qc::api::util;
use Readonly;
use npg_common::roles::run::path_info;

with 'MooseX::Getopt';
with 'npg_common::roles::log';

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $FOLDER_PATH_PREFIX_GLOB_PATTERN => $npg_common::roles::run::path_info::HOST_GLOB_PATTERN . q{/IL*};
#q{/{nfs/sf{9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45},staging}/IL*/};

## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_qc::api::monitoring::Role

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 util

npgqc api util

=cut

has 'util'      => (isa          => 'Object',
                    is           =>  'ro',
                    lazy_build   => 1,
                  );

sub _build_util{
   my $util = npg_qc::api::util->new({
                  useragent => undef,
              });
   return $util;
}

=head2 dbh

DBI dbh from npgqc api util

=cut
has 'dbh'      => (isa          => 'Object',
                   is           =>  'ro',
                   lazy_build   => 1,
                  );

sub _build_dbh{
   my $self = shift;
   return $self->util->dbh();
}

=head2 schema

Dbix class schema from npgqc api util

=cut

has 'schema'     => (isa          => 'Object',
                     is           =>  'ro',
                     lazy_build   => 1,
                    );

sub _build_schema{
   my $self = shift;
   return $self->util->schema();
}

=head2 pattern_prefix

runfolder glob pattern prefix
 
=cut

has 'pattern_prefix'   => (isa          => 'Str',
                           is          => 'rw',
                           default     =>  $FOLDER_PATH_PREFIX_GLOB_PATTERN,
                          );

=head2 glob_pattern

runfolder glob pattern

_build_golb_pattern needed in the sub class
 
=cut

has 'glob_pattern'   => (isa         => 'Str',
                         is          => 'rw',
                         lazy_build  => 1,
                        );

=head2 runlist_db

id_run list already in the database

_build_runlist_db needed in the sub class
 
=cut
has 'runlist_db' => (is          => 'rw',
                     isa         => 'HashRef',
                     lazy_build  => 1,
                    );

=head2 runfolder_list_todo

run folder list with required file inside
 
=cut
has 'runfolder_list_todo' => (is          => 'rw',
                              isa         => 'ArrayRef',
                              lazy_build  => 1,
                             );

sub _build_runfolder_list_todo {
  my $self = shift;

  my @runfolder_list_todo = ();

  my $pattern_to_glob = $self->pattern_prefix().q{/}.$self->glob_pattern();

  my @file_list = < $pattern_to_glob >;

  my %fs_inode_hash;

  foreach my $file (@file_list){

    my $file_stat = join q(,), stat $file;
    my ( $run_folder )  = $file =~ /^(.*\/\d+_(IL|HS)\d+_\d+\w*\/)/mxs;

    if( $run_folder && ! $fs_inode_hash {$file_stat} ) {
       my ($type, $id_run ) = $run_folder =~ /\/\d+_(IL|HS)\d+_(\d+)/mxs;
       $id_run += 0;
       if($id_run && ! $self->runlist_db->{$id_run}){
         push @runfolder_list_todo, $run_folder;
       }
       $fs_inode_hash {$file_stat }++;
    }
  }
  return \@runfolder_list_todo;
}

=head2 run

get the list of runfolder with the required file
save them into database if not there
 
=cut
sub run {
  my $self = shift;

  return 1;
}

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item MooseX::Getopt

=item Carp

=item English qw{-no_match_vars}

=item npg_qc::api::util

=item npg_common::roles::log

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
