#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-01-27
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: Runinfo.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/api/monitoring/Runinfo.pm $
#

package npg_qc::api::monitoring::Runinfo;

use strict;
use warnings;
use Moose;
use Carp;
use English qw{-no_match_vars};
use npg_qc::api::util;
use npg_qc::api::loader::Runinfo;
use Readonly;
use npg_qc::api::run_tile;

with 'npg_qc::api::monitoring::Role';

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $GLOB_PATTERN => q{{outgoing,analysis,incoming}/*/RunInfo.xml};

## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_qc::api::monitoring::Runinfo

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 _build_glob_pattern

lazy build method for glob_pattern defined in the role
 
=cut

sub _build_glob_pattern {

   return $GLOB_PATTERN;
}

=head2 _build_runlist_db

lazy build method

id_run list already in the database
 
=cut

sub _build_runlist_db {
   my $self = shift;
   my @list = $self->schema->resultset('RunInfo')
         ->search(undef,{columns => [qw/id_run/],});

   my %runlist = map { $_->id_run() => 1; } @list;

   return \%runlist;
}


=head2 run
 
=cut
sub run {
  my $self = shift;

  $self->log('Finding runfolder with RunInfo.xml file and load them into QC database');
  $self->log('There are '.scalar @{$self->runfolder_list_todo}.' runs to do' );
  foreach my $runfolder (@{$self->runfolder_list_todo}){
     my $schema = $self->schema();
     my $loader = npg_qc::api::loader::Runinfo->new({
                            runfolder_path => $runfolder,
                            schema         => $schema,
                            });
     eval{
       $loader->run();
       1;
     } or do {
       carp $EVAL_ERROR;
     };
  }

  return 1;
}

no Moose;
__PACKAGE__->meta->make_immutable;

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item MooseX::Getopt

=item Carp

=item English qw{-no_match_vars}

=item npg_qc::api::util

=item npg_qc::api::loader::Runlog

=item npg_qc::api::monitoring::Role

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
