#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-01-27
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: Runlog.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/api/monitoring/Runlog.pm $
#

package npg_qc::api::monitoring::Runlog;

use strict;
use warnings;
use Moose;
use Carp;
use English qw{-no_match_vars};
use npg_qc::api::util;
use npg_qc::api::loader::Runlog;
use Readonly;
use npg_qc::api::run_tile;

with 'npg_qc::api::monitoring::Role';

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $GLOB_PATTERN => q{{outgoing,analysis}/*/RunLog_concat.xml};

## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_qc::api::monitoring::Runlog

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 _build_glob_pattern

lazy build method for glob_pattern defined in the role
 
=cut

sub _build_glob_pattern {

   return $GLOB_PATTERN;
}

=head2 _build_runlist_db

lazy build method

id_run list already in the database
 
=cut

sub _build_runlist_db {
   my $self = shift;

   $self->log( q{checking any run in the database with run_tiles with row numbers} );
   my $util = $self->util();
   my $run_tile_obj = npg_qc::api::run_tile->new({util => $util});
   my $movez_id_runs = $run_tile_obj->movez_id_runs();
   if($movez_id_runs){
      my $num_run_in_db = scalar keys %{$movez_id_runs};
      $self->log( "there are $num_run_in_db runs in the database with number of tiles which rows not null");
    }else{
      $movez_id_runs = {};
      $self->log( q{there is no id_run in the database with row not null} );
    }
   return $movez_id_runs;
}


=head2 run
 
=cut
sub run {
  my $self = shift;

  $self->log('Finding runfolder with RunLog_concat.xml file and load them into QC database');
  $self->log('There are '.scalar @{$self->runfolder_list_todo}.' runs to do' );

  foreach my $runfolder (@{$self->runfolder_list_todo}){
     my $util = $self->util();
     my $loader = npg_qc::api::loader::Runlog->new({
                            runfolder_path => $runfolder,
                            util           => $util,
                            });
     eval{
       $loader->run();
       1;
     } or do {
       carp $EVAL_ERROR;
     };
  }

  return 1;
}

no Moose;
__PACKAGE__->meta->make_immutable;

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item MooseX::Getopt

=item Carp

=item English qw{-no_match_vars}

=item npg_qc::api::util

=item npg_qc::api::loader::Runlog

=item npg_qc::api::monitoring::Role

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
