#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-09-02
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: DB.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/autoqc/db/DB.pm $
#

package npg_qc::autoqc::db::DB;

use strict;
use warnings;
use JSON;
use Carp;
use English qw{-no_match_vars};
use Fey::Object::Iterator::FromSelect;
use Moose::Role;
use File::Basename;

use npg_qc::autoqc::qc_store::options qw/$ALL $LANES $PLEXES/;
use npg_qc::autoqc::db::Schema;
use npg_qc::autoqc::results::collection;
use npg_qc::autoqc::autoqc;

use npg::api::run;


our $VERSION   = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };
#:
sub save_from_json{
  my ($self, $json, $id_run, $position) = @_;

  my $package_from_json = $self->_get_package_from_json($json);

  my $package_file_name = $self->_package2filename($package_from_json);
  require  $package_file_name;

  my $obj = $package_from_json->thaw($json);

  if( $id_run && !$obj->id_run() ){
    $obj->id_run($id_run);
  }

  if( $position && !$obj->position() ){
    $obj->position($position);
  }

  my $db_class_name = $self->_result_class2db_class($package_from_json);
  my $db_class_file_name = $self->_package2filename($db_class_name);

  require  $db_class_file_name;

  return $db_class_name->save($obj, $package_from_json);
}

sub save_from_jsonfile{
  my ($self, $json_file) = @_;

  my $json;

  open my $JSON_FILE, q{<}, $json_file or croak "Couldn't open file: $json_file";

  while ( my $json_line = <$JSON_FILE>){

    chomp $json_line;
    $json .= $json_line;
  }

  close $JSON_FILE or croak "Couldn't close file: $json_file";

  my ($id_run, $position) = $self->_get_id_run_position_from_filename($json_file);

  return $self->save_from_json($json, $id_run, $position);
}

sub save{
  my ($self, $obj, $class_name) = @_;

  my $values = $obj->pack();

  if(! $class_name){

    $class_name = $values->{__CLASS__};
    $class_name =~ s/-\d*$//mxs;
  }

  my $db_class_name = $self->_result_class2db_class($class_name);
  my $db_class_file_name = $self->_package2filename($db_class_name);

  require  $db_class_file_name;
  delete $values->{__CLASS__};

  return $db_class_name->insert(%{$values});
}

sub _get_package_from_json{
  my ($self, $json) = @_;

  my $json_hashref = from_json($json);
  my $class = $json_hashref->{__CLASS__};
  $class =~ s/-\d*$//mxs;
  return $class;
}

sub _package2filename{
  my ($self, $package_name) = @_;

  $package_name =~ s{::}{/}gxms;

  return  $package_name . q{.pm};
}

sub _result_class2db_class{
  my ($self, $result_class) = @_;

  $result_class =~ s{results}{db::checks}gxms;
  return $result_class;
}

sub lane_count_per_run_db{
  my ($self, $qc_check_name, $id_run) = @_;

  my $dbh = npg_qc::autoqc::db::Schema->DBIManager()->default_source()->dbh();

  $qc_check_name = lc $qc_check_name;

  my $count_lane_per_run;
  eval {
     my $query = qq{SELECT COUNT(*) FROM $qc_check_name WHERE id_run = ?};
     my $sth = $dbh->prepare($query);

     $sth->execute($id_run);
     $count_lane_per_run = $sth->fetchrow_arrayref()->[0];
     1;
  } or do {
     croak $EVAL_ERROR;
  };
  return $count_lane_per_run;
}

sub list_per_run{
   my ($self, $id_run, $qc_check_name, $lanes, $what) = @_;

   if(!$id_run){
     $id_run = $self->id_run();
   }

   $what  = (!defined $what || ($what != $ALL && $what != $PLEXES)) ? $LANES : $what;
   $lanes = (defined $lanes) ? $lanes : [];

   my $qc_check_table;
   my $check_package_name;

   if($qc_check_name){

     $check_package_name = qq{npg_qc::autoqc::db::checks::$qc_check_name};
     require $self->_package2filename($check_package_name);
     $qc_check_table = $qc_check_name;
   }else{

     ($qc_check_table) = $self =~ /::(\w+)$/mxsi;
     $check_package_name = $self;
   }

   $qc_check_table = lc $qc_check_table;

   my $select_per_run = $check_package_name->SchemaClass()->SQLFactoryClass()->new_select();

   my $table =  $check_package_name->Table();
   $select_per_run = $select_per_run->select( $table )->from( $table );

   $select_per_run = $select_per_run->where( $table->column(q{id_run}), q{=}, $id_run );

   # the default value is a number and is returned as a number by mysql, but sqlite returns it as a string
   # the only common method between Fey::Literal::String and Fey::Literal::Number is id() that in both
   # cases returns the value itself; hence we are using ->id()
   my $no_tag = $table->column(q{tag_index})->default()->id();

   # the string that is returned by sqlite is in double quotes; so we need to extract the number
   # which is -1 at the moment
   ($no_tag) = $no_tag =~ /(-?\d+)/sxm;

   if ($what == $LANES) {
       $select_per_run = $select_per_run->where( $table->column(q{tag_index}), q{=}, $no_tag);
   } elsif ($what == $PLEXES) {
       $select_per_run = $select_per_run->where( $table->column(q{tag_index}), q{!=}, $no_tag);
   }

   if (@{$lanes}) {
       $select_per_run = $select_per_run->where( $table->column(q{position}), 'IN', @{$lanes} );
   }

   if ($what == $LANES) {
       $select_per_run = $select_per_run->order_by( $table->column(q{position}), q{ASC} );
   } else {
       $select_per_run = $select_per_run->order_by( $table->column(q{position}), q{ASC},
                                                    $table->column(q{tag_index}), q{ASC} );
   }

   my $dbh = $check_package_name->_dbh($select_per_run);

   my $object_list =
          Fey::Object::Iterator::FromSelect->new
          ( classes     => $check_package_name->meta()->ClassForTable( $table ),
            dbh         => $dbh,
            select      => $select_per_run,
            bind_params => [ $select_per_run->bind_params() ],
          );

   my @object_array = ();

   while(my $object = $object_list->next()) {
       push @object_array, $object;
   }

   return \@object_array;
}

sub autoqc_data_by_run_db{
  my ($self, $id_run, $lanes, $what) = @_;

  my $collection = npg_qc::autoqc::results::collection->new();

  my $checks_list = npg_qc::autoqc::autoqc->checks_list();

  foreach my $qc_check_name (@{$checks_list}){

    my $object_list_db = $self->list_per_run($id_run, $qc_check_name, $lanes, $what);
    $collection->add( $object_list_db );
  }
  return $collection;
}

sub count_archived_list {
   my ( $self, $id_run) = @_;

   my $archived_list = $self->autoqc_data_by_run_db($id_run, undef, $ALL)->results();

   my $list_by_lane       = {};
   my $list_by_lane_split = {};
   my $list_by_lane_tag   = {};
   my $list_by_lane_tag_split = {};

   foreach my $qc_rs (@{$archived_list}){

       my $position   = $qc_rs->position();
       my $class_name = $qc_rs->class_name();
       my $tag_index  = $qc_rs->tag_index();

       my $subtype_method = $self->_autoqc_subtype_method_name($class_name);
       my $subtype;
       if($subtype_method){
           $subtype = $qc_rs->$subtype_method();
       }

       if( !defined $tag_index ){
           if( !$subtype || ( $class_name eq q{bam_flagstats} && $subtype eq q{all} )){
               $list_by_lane->{$class_name}->{$position}++;
           }else{
               $list_by_lane_split->{$class_name}->{$position}->{$subtype}++;
           }
       }else{
           if(!$subtype || ( $class_name eq q{bam_flagstats} && $subtype eq q{all} ) ){
               $list_by_lane_tag->{$class_name}->{$position}->{$tag_index}++;
           }else{
               $list_by_lane_tag_split->{$class_name}->{$position}->{$tag_index}->{$subtype}++;
           }
       }

   }

   return { list_by_lane           => $list_by_lane,
            list_by_lane_split     => $list_by_lane_split,
            list_by_lane_tag       => $list_by_lane_tag,
            list_by_lane_tag_split => $list_by_lane_tag_split,
          };
}

sub _autoqc_subtype_method_name {
    my ($self, $class_name) = @_;

    my $method_name = $class_name eq q{split_stats}    ? q{ref_name}
                    : $class_name eq q{bam_flagstats}  ? q{human_split}
                    : $class_name eq q{sequence_error} ? q{sequence_type}
                    : undef;
    return $method_name;
}

sub npg_run_fully_archived {
  my ($self, $id_run) =  @_;

  my $run_api  = npg::api::run->new({id_run => $id_run,
				                        });
  my @lanes = ();
  my %spiked_phix_lanes = ();
  my %non_consented_lanes = ();
  my %tag_list_by_lane = ();

  my $is_paired_read = $run_api->is_paired_read();

  foreach my $lane ( @{$run_api->run_lanes()} ){

     my $position = $lane->position();
     push @lanes, $position;
     if($lane->is_spiked_phix()){
        $spiked_phix_lanes{$position} = 1;
     }

     if( $lane->contains_unconsented_human() ){
        $non_consented_lanes{$position} = 1;
     }

     my $tag_list = $lane->st_lane->tag_list();
     my @tag_index_list = ();
     foreach my $tag (@{$tag_list}){
       push @tag_index_list, $tag->index();
     }
     if(scalar @tag_index_list){
        push @tag_index_list, 0;
        $tag_list_by_lane{$position} = \@tag_index_list;
     }
  }

  my $args_ref = {
     id_run              => $id_run,
     is_paired_read      => $is_paired_read,
     lanes               => \@lanes,
     tag_list_by_lane    => \%tag_list_by_lane,
     spiked_phix_lanes   => \%spiked_phix_lanes,
     non_consented_lanes => \%non_consented_lanes,
  };

  return $self->run_fully_archived($args_ref);
}

sub run_fully_archived {
    my ( $self, $args_ref ) = @_;

    my $fully_archived = 1;

    my $id_run               = $args_ref->{id_run};
    my $is_paired_read       = $args_ref->{is_paired_read};

    my $count_archived_list  = $self->count_archived_list($id_run);

    my $checks_list = npg_qc::autoqc::autoqc->checks_list();

    foreach my $qc_check_name (@{$checks_list}){

        if(!$is_paired_read && ($qc_check_name eq q{insert_size}) ){
           next;
        }

        $args_ref->{list_by_lane}           = $count_archived_list->{list_by_lane}->{$qc_check_name};
        $args_ref->{list_by_lane_split}     = $count_archived_list->{list_by_lane_split}->{$qc_check_name};
        $args_ref->{list_by_lane_tag}       = $count_archived_list->{list_by_lane_tag}->{$qc_check_name};
        $args_ref->{list_by_lane_tag_split} = $count_archived_list->{list_by_lane_tag_split}->{$qc_check_name};

        $args_ref->{check_name}             = $qc_check_name;

        if( !$self->_one_check_data_completeness($args_ref) ){
            $fully_archived = 0;
        }

    }

    return $fully_archived;
}

sub _one_check_data_completeness { ## no critic (Subroutines::ProhibitExcessComplexity)
    my ($self, $args_ref) = @_;

    my $complete = 1;

    my $lanes                = $args_ref->{lanes};
    my $tag_list_by_lane     = $args_ref->{tag_list_by_lane};
    my $spiked_phix_lanes    = $args_ref->{spiked_phix_lanes};
    my $non_consented_lanes  = $args_ref->{non_consented_lanes};

    my $list_by_lane            = $args_ref->{list_by_lane};
    my $list_by_lane_split      = $args_ref->{list_by_lane_split};
    my $list_by_lane_tag        = $args_ref->{list_by_lane_tag};
    my $list_by_lane_tag_split  = $args_ref->{list_by_lane_tag_split};

    my $check_name           = $args_ref->{check_name};

    foreach my $lane ( @{$lanes} ){

        #get tag list for the lane
        my $tag_list_one_lane = $tag_list_by_lane->{$lane};
        $tag_list_one_lane ||= [];

        #non consented lanes 
        if( $non_consented_lanes->{$lane} ){

           #check nonconsented lane split stats
           if( ( $check_name eq q{split_stats} )
                 && ( !$list_by_lane_split->{$lane}->{human} ) ){

              $complete = 0;
              print "Nonconsented split stats lane $lane missing\n" or croak 'can not print';
           }

           #check nonconsented lane bam_flagstats for human and nonhuman part
           if( ( $check_name eq q{bam_flagstats} )
                && ( !$list_by_lane_split->{$lane}->{nonhuman} || !$list_by_lane_split->{$lane}->{human} )
                && (scalar @ {$tag_list_one_lane} == 0 ) ){

             $complete = 0;
             print "human or nonhuman bam flagstats lane $lane missing\n" or croak 'can not print';
           }

           #check all other lane level data
           if( ( $check_name ne q{split_stats} )
              && ( $check_name ne q{bam_flagstats} )
              && ( $check_name ne q{tag_decode_stats} )
              && (! $list_by_lane->{$lane} ) ){

              if( $check_name ne q{gc_bias} ){
                $complete = 0;
              }
              print "$check_name lane $lane missing\n" or croak 'can not print';
           }

        }else {

           #check all other lane level data without nonconsented data
           if( ( $check_name ne q{split_stats} )
              && ( $check_name ne q{tag_decode_stats} )
              && (! $list_by_lane->{$lane} ) ){

              if( ($check_name ne q{gc_bias}) && ($check_name ne q{bam_flagstats}) ){
                $complete = 0;
              }

              if( ($check_name eq q{bam_flagstats}) && (scalar @ {$tag_list_one_lane} == 0 ) ){
                $complete = 0;
              }
              print "$check_name lane $lane missing\n" or croak 'can not print';
           }
        }

        #check tag decode stats if multiplexed       
        if( (scalar @{$tag_list_one_lane})
            && ( $check_name eq q{tag_decode_stats} )
            && (! $list_by_lane->{$lane} ) ){

              $complete = 0;
              print "tag decode stats lane $lane missing" or croak 'can not print';
         }

        #no plex level data for tag_decode_stats and split_stats
        if( ( $check_name eq q{tag_decode_stats} )
          ||( $check_name eq q{split_stats} ) ){

           $tag_list_one_lane = [];
        }

        #check plex level data
        foreach my $tag_index (@{$tag_list_one_lane}){

            if( ( ( $check_name ne q{bam_flagstats} ) || ( ( $check_name eq q{bam_flagstats} ) && (! $non_consented_lanes->{$lane} ) ) )
                && (! $list_by_lane_tag->{$lane}->{$tag_index} ) ) {

                if( $check_name ne q{gc_bias} ){
                  $complete = 0;
                }
                print "$check_name lane $lane tag $tag_index missing\n" or croak 'can not print';
            }

            if( ( $check_name eq q{bam_flagstats} )
                && ( $non_consented_lanes->{$lane} )
                && (!$list_by_lane_tag_split->{$lane}->{$tag_index}->{nonhuman} || !$list_by_lane_tag_split->{$lane}->{$tag_index}->{human}) ) {

               $complete = 0;
               print "$check_name human or nonhumam lane $lane tag $tag_index missing\n" or croak 'can not print';
            }
        }

        #check spiked phix sequence_error, bam flag stats and split_stats
        my $spiked_phix_lane = $spiked_phix_lanes->{$lane};
        if( $spiked_phix_lane
            && ( $check_name eq q{sequence_error} || $check_name eq q{bam_flagstats} || $check_name eq q{split_stats} )
            && ( !$list_by_lane_split->{$lane}->{phix} ) ){

            $complete = 0;
            print "$check_name lane $lane spiked phix missing\n" or croak 'can not print';
        }

    }

    return $complete;
}

sub _get_id_run_position_from_filename{
  my ($self, $filename) = @_;

  my ($name, $directories, $suffix) = fileparse($filename, qr{.json}mxs);
  my ($id_run, $position) = $name =~ /^(\d+)_(\d)/mxs;

  return ($id_run, $position);
}

1;
__END__

=head1 NAME

npg_qc::autoqc::db::DB

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

=head1 DESCRIPTION

This role provides some saving methods to save data to the database
from any Moose objects, JSON strings or JSON files using their corresponding Fey::ORM table class.
And also some other methods related to the database

=head1 SUBROUTINES/METHODS

=head2 _get_package_from_json - get the package name from json string

=head2 _package2fileanme - transfer perl package name into package file name
in order to require this package

=head2 _result_class2db_class - transfer result class name into its corresponding db class

=head2 save - give an Moose object to save its data into database

=head2 save_from_json - give a JSON string, covert into back to a Moose object then call the save method

=head2 save_from_jsonfile - give a JSON file, covert into back to a Moose object then call the save method

=head2 lane_count_per_run_db - given id_run and qc_check_name, check how many lanes data in the database

=head2 list_per_run - given id_run and qc_check_name, return a list of objects from database, this method can be called from its subclass Fey table class without qx_check_name or id_run

=head2 autoqc_data_by_run_db - return a collection of autoqc data from database as object list

=head2 count_archived_list - count each record by check_name, lane, tag and subtype

=head2 run_fully_archived - given a list of lanes in arraryref, spiked_phix lanes and nonconsented lanes  in hashref, check their auto data fully archived in the database

=head2 npg_run_fully_archived - given a run_id, use npg api run to get most of run information and then check archived autoqc data

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item JSON

=item Carp

=item English

=item npg_qc::autoqc::db::Schema

=item npg_qc::autoqc::results::collection

=item Fey::Object::Iterator::FromSelect

=item npg_qc::autoqc::autoqc

=item npg::api::run

=item Moose::Role

=item File::Basename;

=item npg_qc::autoqc::qc_store::options qw/$ALL $LANES $PLEXES/;

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
