#########
# Author:        jo3
# Maintainer:    $Author: ajb $
# Created:       2009-10-08
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: gc_bias.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/autoqc/db/checks/gc_bias.pm $
#

package npg_qc::autoqc::db::checks::gc_bias;

use strict;
use warnings;
use npg_qc::autoqc::db::Schema;
use Fey::ORM::Table;
use JSON;

with qw( npg_qc::autoqc::role::gc_bias npg_qc::autoqc::db::DB);

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };


sub load_table {    ## no critic (Subroutines::RequireArgUnpacking)
    my ( $self, $schema ) = @_;

    my $table = $schema->table('gc_bias');

    if ( !$self->Table() ) {
        has_table $table;

        my $column = $self->Table()->column('tag_index');
        my $no_tag = $column ->default()->id();
        ($no_tag) = $no_tag =~ /(-?\d+)/sxm;

        transform 'actual_quantile_x'
            => inflate { defined $_[1] ? from_json( $_[1] ) : undef }
            => deflate { defined $_[1] ? to_json( $_[1] )   : $_[1] };

        transform 'actual_quantile_y'
            => inflate { defined $_[1] ? from_json( $_[1] ) : undef }
            => deflate { defined $_[1] ? to_json( $_[1] )   : $_[1] };

        transform 'gc_lines'
            => inflate { defined $_[1] ? from_json( $_[1] ) : undef }
            => deflate { defined $_[1] ? to_json( $_[1] )   : $_[1] };

        transform 'plot_x'
            => inflate { defined $_[1] ? from_json( $_[1] ) : undef }
            => deflate { defined $_[1] ? to_json( $_[1] )   : $_[1] };

        transform 'plot_y'
            => inflate { defined $_[1] ? from_json( $_[1] ) : undef }
            => deflate { defined $_[1] ? to_json( $_[1] )   : $_[1] };

        transform 'ideal_lower_quantile'
            => inflate { defined $_[1] ? from_json( $_[1] ) : undef }
            => deflate { defined $_[1] ? to_json( $_[1] )   : $_[1] };

        transform 'ideal_upper_quantile'
            => inflate { defined $_[1] ? from_json( $_[1] ) : undef }
            => deflate { defined $_[1] ? to_json( $_[1] )   : $_[1] };

        transform 'info'
            => inflate { defined $_[1] ? from_json( $_[1] ) : undef }
            => deflate { defined $_[1] ? to_json( $_[1] )   : $_[1] };

        transform 'tag_index'
           => deflate { defined $_[1] ? $_[1]    : $no_tag }
           => inflate { $_[1] != $no_tag ? $_[1] : undef   };
    }
    return $self->Table();
}


1;

__END__


=head1 NAME

npg_qc::autoqc::db::checks::gc_bias

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

=head1 DESCRIPTION

This class defines a database table class using Fey::ORM, linking to autoqc
data table contamination and inheriting the autoqc result Moose class and autoqc db
DB class to use their methods.

=head1 SUBROUTINES/METHODS

=head2 load_table - load table using given schema

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Fey::ORM::Table

=item JSON

=item npg_qc::autoqc::db::DB

=item npg_qc::autoqc::db::Schema

=item npg_qc::autoqc::role::gc_bias;

=item strict

=item warnings

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

John O'Brien, jo3@sanger.ac.uk

=head1 LICENSE AND COPYRIGHT


=cut
