#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2009-11-18
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: sequence_error.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/autoqc/db/checks/sequence_error.pm $
#
package npg_qc::autoqc::db::checks::sequence_error;


use strict;
use warnings;
use npg_qc::autoqc::db::Schema;
use Carp;
use Fey::ORM::Table;
use JSON;

with qw( npg_qc::autoqc::role::sequence_error  npg_qc::autoqc::db::DB );

our $VERSION   = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };

sub load_table { ## no critic ( Subroutines::RequireArgUnpacking Subroutines::ProhibitExcessComplexity )
  my ($self, $schema) = @_;

  my $table = $schema->table('sequence_error');

  if(!$self->Table()){

     has_table $table;
     my $column = $self->Table()->column('tag_index');
     my $no_tag = $column ->default()->id();
     ($no_tag) = $no_tag =~ /(-?\d+)/sxm;

     transform 'forward_errors'
       => inflate { defined $_[1] ? from_json($_[1] ) : undef }
       => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

     transform 'reverse_errors'
       => inflate { defined $_[1] ? from_json($_[1] ) : undef }
       => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

     transform 'forward_count'
       => inflate { defined $_[1] ? from_json($_[1] ) : undef }
       => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

     transform 'reverse_count'
       => inflate { defined $_[1] ? from_json($_[1] ) : undef }
       => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

     transform 'forward_n_count'
       => inflate { defined $_[1] ? from_json($_[1] ) : undef }
       => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

     transform 'reverse_n_count'
       => inflate { defined $_[1] ? from_json($_[1] ) : undef }
       => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

     transform 'info'
       => inflate { defined $_[1] ? from_json($_[1] ) : undef }
       => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

     transform 'tag_index'
       => deflate { defined $_[1] ? $_[1]    : $no_tag }
       => inflate { $_[1] != $no_tag ? $_[1] : undef   };

     transform 'sequence_type'
       => deflate { defined $_[1] ? $_[1]    : 'default' }
       => inflate { $_[1] ne 'default' ? $_[1] : undef   };

     transform 'reverse_quality_bins'
       => inflate { defined $_[1] ? from_json($_[1] ) : undef }
       => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

     transform 'forward_quality_bins'
       => inflate { defined $_[1] ? from_json($_[1] ) : undef }
       => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

     transform 'quality_bin_values'
       => inflate { defined $_[1] ? from_json($_[1] ) : undef }
       => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

  }
  return $self->Table();
}

sub error_per_run {
  my ($self, $id_run) = @_;

  if(!$id_run){

    $id_run = $self->id_run();
  }

  return __PACKAGE__->list_per_run($id_run);
}

1;
__END__

=head1 NAME

npg_qc::autoqc::db::checks::sequence_error

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

=head1 DESCRIPTION

This class defines a database table class using Fey::ORM,
linking to autoqc data table qx_yield,
inheriting autoqc result Moose class and autoqc db DB class to use their methods.

=head1 SUBROUTINES/METHODS

=head2 load_table - load table using given schema

=head2 error_per_run - given id_run, return a list of error objects in arrayref

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict

warnings

npg_qc::autoqc::db::Schema

Fey::ORM::Table

npg_qc::autoqc::role::sequence_error

npg_qc::autoqc::db::DB

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
