#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-09-24
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: split_stats.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/autoqc/db/checks/split_stats.pm $
#
package npg_qc::autoqc::db::checks::split_stats;

use strict;
use warnings;
use npg_qc::autoqc::db::Schema;
use Fey::ORM::Table;
use JSON;
use npg_qc::autoqc::db::checks::split_stats_coverage;

with qw( npg_qc::autoqc::role::split_stats  npg_qc::autoqc::db::DB );

our $VERSION   = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };


sub load_table{ ## no critic ( Subroutines::RequireArgUnpacking )
  my ($self, $schema) = @_;

  my $table = $schema->table('split_stats');

  if(!$self->Table()){

    has_table $table;

    my $column = $self->Table()->column('tag_index');
    my $no_tag = $column ->default()->id();
    ($no_tag) = $no_tag =~ /(-?\d+)/sxm;

    transform 'alignment_depth1'
      => inflate { defined $_[1] ? from_json($_[1] ) : undef }
      => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

    transform 'alignment_depth2'
      => inflate { defined $_[1] ? from_json($_[1] ) : undef }
      => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

    transform 'info'
      => inflate { defined $_[1] ? from_json($_[1] ) : undef }
      => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

    transform 'tag_index'
       => deflate { defined $_[1] ? $_[1]    : $no_tag }
       => inflate { $_[1] != $no_tag ? $_[1] : undef   };

    my $coverage_table = $schema->table( 'split_stats_coverage' );

    # has_many 'split_stats_coverages' => ( table =>  $coverage_table );

    # getting foreign key info from the database very slow sometimes
    # to disable this, Fey::Loader::mysql->_fk_info_sth needs return undef
    # when foreign key info not availabe for these two tables, using folowing instead
    my $select = $self->SchemaClass()->SQLFactoryClass()->new_select();
    $select->select( $coverage_table )->from( $coverage_table )
           ->where( $coverage_table->column('id_split_stats'), q{=}, Fey::Placeholder->new() );

    has_many 'split_stats_coverages' =>
                   ( table       => $coverage_table,
                     select      => $select,
                     bind_params => sub { $_[0]->id_split_stats() },
                   );
  }

  return $self->Table();
}

sub coverages_in_hash{
  my $self = shift;

  my $coverage_both = $self->split_stats_coverages();
  my $coverages;

  while(my $coverage_obj = $coverage_both->next()){

      my $chromosome = $coverage_obj->chromosome();
      my $fastq = $coverage_obj->fastq();
      my $coverage = $coverage_obj->coverage();
      $coverages->{$fastq}->{$chromosome} = $coverage;
  }

  return $coverages;
}

sub save{
  my ($self, $obj) = @_;

  my $values = $obj->pack();

  my $alignment_coverage1 = $values->{alignment_coverage1};
  my $alignment_coverage2 = $values->{alignment_coverage2};

  delete $values->{__CLASS__};
  delete $values->{alignment_coverage1};
  delete $values->{alignment_coverage2};

  my $new_db_obj = __PACKAGE__->insert(%{$values});

  my $id_split_stats =  $new_db_obj->id_split_stats();
  $self->save_alignment_coverage( $alignment_coverage1, $id_split_stats , 1);
  $self->save_alignment_coverage( $alignment_coverage2, $id_split_stats , 2);

  return $new_db_obj;
}

sub save_alignment_coverage{
  my ($self, $alignment_coverage, $id_split_stats, $fastq) = @_;

  foreach my $chromsome (keys %{$alignment_coverage}){

    my %values = (chromosome => $chromsome, id_split_stats => $id_split_stats, fastq =>  $fastq);
    $values{coverage} = $alignment_coverage->{$chromsome};
    npg_qc::autoqc::db::checks::split_stats_coverage->insert(%values);
  }

  return 1;
}

1;
__END__

=head1 NAME

npg_qc::autoqc::db::checks::split_stats

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

=head1 DESCRIPTION

This class defines a database table class using Fey::ORM,
linking to autoqc data table split_stats,
inheriting autoqc result Moose class and autoqc db DB class to use their methods.

=head1 SUBROUTINES/METHODS

=head2 load_table - load table using given schema

=head2 split_stats_coverages - return a list of alignment coverages related to this splitting

=head2 coverages_in_hash - covert the list of alignment coverages into the hash like original results

=head2 save - override the save method to save the alignment coverages in a separate table

=head2 save_alignment_coverage - save related alignment coverages

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict;
warnings;
npg_qc::autoqc::db::Schema;
Fey::ORM::Table;
npg_qc::autoqc::role::split_stats;
npg_qc::autoqc::db::DB;
npg_qc::autoqc::db::checks::split_stats_coverage;

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
