#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       29 October 2009
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: result.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/autoqc/role/result.pm $
#

package npg_qc::autoqc::role::result;

use strict;
use warnings;
use Moose::Role;
use Carp;
use English qw(-no_match_vars);
use MooseX::Storage;

with Storage( 'traits' => ['OnlyWhenBuilt'],
              'format' => 'JSON',
              'io'     => 'File' ), 'npg_qc::autoqc::role::rpt_key';

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/smx; $r; };
## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_qc::autoqc::role::result

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 class_name

Name of the class that did the check

=cut
sub class_name {
    my $self = shift;
    my ($ref) = (ref $self) =~ /(\w*)$/smx;
    return $ref;
}


=head2 package_name

Name of the package that did the check

=cut
sub package_name {
    my $self = shift;
    return (ref $self);
}


=head2 check_name

Human readable check name

=cut
sub check_name {
    my $self = shift;
    my $name = $self->class_name;
    $name =~ s/sequence_error/sequence mismatch/xms;
    if ($self->can(q[sequence_type]) && $self->sequence_type) {
        $name .= q[ ] . $self->sequence_type;
    }
    $name =~ s/_/ /gsmx;
    return $name;
}


=head2 equals_byvalue

Determines whether the values of the attributes in the object are as listed in the argument hash. 
Takes a reference to a hash where keys are the names of the attributes and values are expected values. 
Returns true if all values are as expected, otherwise returns false.
Supports comparison on the following attributes: position, check_name, class_name, id_run.

 my $r = npg_qc::autoqc::results::result->new({id_run => 222, position => 2, path => q[my_path]});
 $r->equals_byvalue({id_run => 222, position => 2,}); #returns 1
 $r->equals_byvalue({id_run => 222, position => 1,}); #returns 0
 $r->equals_byvalue({id_run => 222,});                #returns 1

=cut
sub equals_byvalue {
    my ($self, $h) = @_;
    my @keys =keys %{$h};
    if (@keys == 0) {croak q[No parameters for comparison];}
    foreach my $key (@keys) {
        if ($key !~ /position|class_name|check_name|id_run|tag_index/smx) {
            croak qq[Value of the $key attribute cannot be compared. Comparable attributes: position, check_name, class_name, id_run, tag_index];
	}
        if ($key eq q[tag_index]) {
            if (!defined $h->{$key} && !defined $self->$key) { next; }
            if (!defined $h->{$key} && defined $self->$key || defined $h->{$key} && !defined $self->$key) {return 0;}
	}
        if ($self->$key ne $h->{$key}) {return 0;}
    }
    return 1;
}

=head2 add_comment

Appends a comment to a string of comments
  
=cut
sub add_comment {

    my ($self, $comment) = @_;
    if (!$comment) {return 1;}
    if (!$self->comments) {
        $self->comments($comment);
    } else {
        $self->comments($self->comments . q[ ] . $comment);
    }
    return 1;
}

1;
__END__


=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item MooseX::Storage

=item Carp

=item English

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Author: Marina Gourtovaia E<lt>mg8@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Marina Gourtovaia

This file is part of NPG.

NPG is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut
