#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-08-07
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: qcal.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/model/qcal.pm $
#

package npg_qc::model::qcal;
use strict;
use warnings;
use English qw{-no_match_vars};
use Carp;
use base qw(npg_qc::model);

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };

__PACKAGE__->mk_accessors(__PACKAGE__->fields());

sub fields {
  return qw(
            id_qcal
            id_run
            position
            end
            twenty
            twentyfive
            thirty
          );
}

sub init {
  my $self = shift;

  if($self->id_run() &&
     $self->end &&
     $self->position() &&
     !$self->id_qcal()) {

    my $query = q(SELECT id_qcal
                  FROM qcal
                  WHERE id_run = ?
                  AND end = ?
                  AND position = ?
                 );

    my $ref   = [];

    eval {
      $ref = $self->util->dbh->selectall_arrayref($query, {}, $self->id_run(), $self->end(), $self->position());
      1;
    } or do {
      carp $EVAL_ERROR;
      return;
   };

    if(@{$ref}) {
      $self->id_qcal($ref->[0]->[0]);
    }
  }
  return 1;
}

sub greater_than_refactor {
  my ($self, $field) = @_;

  my $query = qq{SELECT SUM($field) FROM qcal WHERE id_run = ? AND end IN ('0', '1', '2')};

  my $dbh = $self->util->dbh();
  my $sth = $dbh->prepare($query);
  $sth->execute($self->id_run());

  return $sth->fetchrow_arrayref()->[0];
}

sub greater_than_q_twenty {
  my $self = shift;
  return $self->greater_than_refactor('twenty');
}

sub greater_than_q_twentyfive {
  my $self = shift;
  return $self->greater_than_refactor('twentyfive');
}

sub greater_than_q_thirty {
  my $self = shift;
  return $self->greater_than_refactor('thirty');
}


sub qcal_by_lanes {
  my ($self) = @_;

  if(!$self->{qcal_by_lanes}){

    my $qcal_by_lanes;

    my $query = q{SELECT position, end, twenty, twentyfive, thirty FROM qcal WHERE id_run = ? AND end IN ('0', '1', '2')};
    my $dbh = $self->util->dbh();
    my $sth = $dbh->prepare($query);
    $sth->execute($self->id_run());
    while(my @row = $sth->fetchrow_array()){
      my $lane = shift @row;
      my $end = shift @row;
      $qcal_by_lanes->{$lane}->{q20}->{$end} =  shift @row;
      $qcal_by_lanes->{$lane}->{q25}->{$end} =  shift @row;
      $qcal_by_lanes->{$lane}->{q30}->{$end} =  shift @row;
    }
    $self->{qcal_by_lanes}= $qcal_by_lanes;
  }

  return $self->{qcal_by_lanes};
}

1;
__END__
=head1 NAME

npg_qc::model::qcal


=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

  my $oQcal = npg_qc::model::qcal->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fields - return array of fields, first of which is the primary key

  my $aFields = $oQcal->fields();

=head2 greater_than_q_thirty - returns the number of bases above a q30 value

  my $iGreaterThanQThirty = $oQcal->greater_than_q_thirty();

=head2 greater_than_q_twenty - returns the number of bases above a q20 value

  my $iGreaterThanQTwenty = $oQcal->greater_than_q_twenty();

=head2 greater_than_q_twentyfive - returns the number of bases above a q25 value

  my $iGreaterThanQTwentyfive = $oQcal->greater_than_q_twentyfive();
  
=head2 qcal_by_lanes - to return all the q20, q25 and q30 values for each lane
  
=head2 init

=head2 greater_than_refactor - refactor of common code from greater_than_q_thirty, greater_than_q_twenty, greater_than_q_twentyfive

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
npg_qc::model
English
Carp

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
