#!/usr/bin/perl

use strict;
use warnings;
use Carp;

use npg_common::extractor::fastq_old2new;

our $FUSE = q[/fuse/mpsafs/runs];
our $STORE = q[/nfs/sf31/fastq_store_mg8];

open(my $fh, q[<], "no_size_info.txt") or die $!;

my $runs = {};

while (my $row = <$fh>) {
  my @numbers = split q[ ], $row;
  my $id_run = $numbers[0];
  my $position = $numbers[1];
  if (exists $runs->{$id_run}) {
    push @{$runs->{$id_run}}, $position;
  } else {
    $runs->{$id_run} = [$position];
  }
}
close $fh;

foreach my $id_run (sort keys %{$runs}) {
  if (-e join(q[/], $FUSE, $id_run, $id_run . q[_s_1.fastq])) {
    carp qq[Converting run $id_run];
    my $dir_out = join(q[/], $STORE, $id_run);
    if (-e $dir_out) {next; }
    `mkdir $dir_out`;
    npg_common::extractor::fastq_old2new->new(id_run => $id_run, lane => $runs->{$id_run},
                                              dir_in => join(q[/], $FUSE, $id_run),
                                              dir_out => $dir_out,
                                             )->convert;
  }
}

1;
