use npg_common::sequence::reference;
use Getopt::Long;
use Cwd qw/cwd/;
use Carp;
use English;

our @CHECK_NAMES = qw/qX_yield insert_size sequence_error adapter gc_fraction/;
our $FUSE = q[/fuse/mpsafs/runs];
our $STORE = q[/nfs/sf31/fastq_store_mg8];
our $SAMPLE_SIZE = 10000;
our $CACHE_DIR_NAME = q[.npg_cache_] . $SAMPLE_SIZE;

sub _get_lanes {
  my $fname = shift;
  my $runs = {};
  open(my $fh, q[<], $fname) or die $!;
  while (my $row = <$fh>) {
    my @numbers = split q[ ], $row;
    my $id_run = $numbers[0];
    my $position = $numbers[1];
 
    if (exists $runs->{$id_run}) {
      push @{$runs->{$id_run}}, $position;
    } else {
      $runs->{$id_run} = [$position];
    }
  }
  close $fh;

  return $runs;
}


################  Main ##############################

my $list_in  = q[with_refs_lanes.txt];
my $runs = _get_lanes($list_in);

local $ENV{PATH} = $ENV{PATH} . q[:/software/solexa/bin];

my $total = 0;

foreach my $id_run (sort keys %{$runs}) {
    if ($total == 5) { last; }
  
  my $store_path = join(q[/], $STORE, $id_run);
  local $ENV{NPG_FASTQ_CACHE} = join(q[/], $store_path, $CACHE_DIR_NAME);

        if (-e $store_path) {
  my $qc_out = join(q[/], $store_path, q[qc]);
  if (!-e $qc_out) { 
    mkdir $qc_out;
  } else {
    next;
  }

  my $qc_in = join(q[/], $FUSE, $id_run);

  $total++;
  foreach my $check_name (@CHECK_NAMES) {
    my $cmd = 'bsub ';
    if ($check_name eq q[insert_size] || $check_name eq q[sequence_error]) {
        $cmd .= '-R"select[mem>4000] rusage[mem=4000]" -M4000000 ';
    }
    my $iarray = '[' . join(q[,], @{$runs->{$id_run}}) . ']';
    $cmd .= '-J qc_' . $check_name . '_' . $id_run . $iarray . q[ ];
    $cmd .= '-o qc_' . $check_name  . '_' . $id_runs  . '-%J-%I.out ';
    $cmd .= '-e qc_' . $check_name  . '_' . $id_runs  . '-%J-%I.err ';
    $cmd .= '-P pipeline ';
    $cmd .= qq['/nfs/users/nfs_m/mg8/working/npg-qc/prerelease-32.0/bin/qc --check $check_name --qc_in $qc_in --qc_out $qc_out --id_run $id_run --position ] . q[$LSB_JOBINDEX'];
    my $out = `$cmd`;
    carp qq[BSUB OUT $out COMMAND: $cmd];
  }
                         }
}

1;
