#!/software/bin/perl -wT
#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-11-12
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: get_summary.pl 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/scripts/get_summary.pl $
#

use strict;
use warnings;
use Carp;
use lib qw(lib /software/solexa/lib/site_perl/);
use SangerPaths qw(core ensembl tracking trackingnpg badger);
use npg_qc::api::summary;
use npg_qc::api::util;
use LWP::UserAgent;

use English qw{-no_match_vars};

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mx; $r; };

$ENV{'PATH'} = join q(:), qw(/bin /usr/bin /usr/local/bin);

main();
0;

sub main {

  #########
  # overide base uri for npg_qc
  my $uri  = q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg_qc/npg_qc/};
  my $util = npg_qc::api::util->new({
    useragent => undef,
    base_uri  => undef,
    parser    => undef,
  });

  my$oSummary;

  eval {
    $oSummary = npg_qc::api::summary->new({util => $util});
    my ($id_run) = $ARGV[0] =~ /(\d+)/xms;
    $oSummary->id_run($id_run);
    $oSummary->read_summary();
    1;
  } or do {
    croak $EVAL_ERROR;
  };

  my $chip_summary  = $oSummary->get_chip_summary();
  my $lane_summaries = $oSummary->get_lane_summaries();
  foreach my $f ($chip_summary->fields()) {
    my $method = 'get_'.$f;
    print "$method : ". $chip_summary->$method()."\n\n";
  } 

  foreach my $ls (@{$lane_summaries->{read_one}}) {
  foreach my $f ($ls->fields()) {
    my $method = 'get_'.$f;
    print "$method : ". $ls->$method()."\n";
  } 
  print "\n\n" 
  }

}

__END__
=head1 NAME

get_summary

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

  A simple script to use npg_qc::api::summary to obtain the summary information for a run

=head1 DESCRIPTION

  prompt> ./get_summary.pl <id_run>

  spits out the chip summary information and the lane summary info from read one of the run requested, read two is available.

=head1 SUBROUTINES/METHODS

=head2 main - is called in the script and does everything

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
npg_qc::api:util
npg_qc::api:summary
Carp
SangerPaths
LWP::UserAgent
English

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
