use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan'; #tests => 13;
use Test::Exception;
use lib qw{t};
use SangerPaths qw(ensembl core badger trackingnpg tracking);
use t::api::util;
use npg::api::util;
use t::useragent;

BEGIN {
  use_ok(q{npg_qc::api::loader::Cluster_Density});
}

my $runfolder_path = qq{t/data/nfs/sf44/IL1/outgoing/080530_IL1_0956};

my $ua_mock = {q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run/956} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run/956.xml}, };
my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
my $npg_api_util = npg::api::util->new({useragent => $ua});
my $util = t::api::util->new({
                dbh => 'dummy',
                npg_api_util => $npg_api_util,
                fixtures => 1,
                useragent => $ua,
                });

{
  my $loader;
  lives_ok { $loader = npg_qc::api::loader::Cluster_Density->new({
                            runfolder_path => $runfolder_path,
                            util => $util,
  }); } q{loader object creation ok};

  isa_ok($loader, q{npg_qc::api::loader::Cluster_Density}, q{$loader});

  isa_ok($loader->util(), q{npg_qc::api::util}, q{$loader->util});

  is($loader->id_run(), '956', 'correct id_run');

  is($loader->raw_xml_file(), q{t/data/nfs/sf44/IL1/outgoing/080530_IL1_0956/Data/reports/NumClusters By Lane.xml}, 'correct raw xml file name');
  is($loader->pf_xml_file(), q{t/data/nfs/sf44/IL1/outgoing/080530_IL1_0956/Data/reports/NumClusters By Lane PF.xml}, 'correct pf xml file name');

  my $cluster_density_by_lane = $loader->parsing_xml($loader->raw_xml_file());
  is($cluster_density_by_lane->{1}->{min}, '619376.000', 'correct value for lane 1 min');
  is($cluster_density_by_lane->{4}->{max}, '196241.200', 'correct value for lane 4 max');
  is($cluster_density_by_lane->{8}->{p50}, '690931.800', 'correct value for lane 8 p50');

  eval{
    $loader->save_to_db({lane=>1, is_pf=>0, min=>9667.00, max=>97777.00, p50=>88888.00});
  };
  is($EVAL_ERROR, q{}, 'no croak when saving one row');

  eval{
    $loader->save_to_db_list($cluster_density_by_lane, 0);
    $loader->util->dbh->commit;
  };
  is($EVAL_ERROR, q{}, 'no croak when saving a list of rows');

  throws_ok { $loader->run(); } qr{Run folder t/data/nfs/sf44/IL1/outgoing/080530_IL1_0956 is not a genuine run in NPG};
}


1;
