use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';
use Test::Exception;
use lib qw{t};
use SangerPaths qw(core);
use t::api::util;

BEGIN {
  use_ok(q{npg_qc::api::loader::Gerald_Summary});
}

my $util = t::api::util->new({
                dbh => 'dummy',
                fixtures => 1,
                useragent => undef,
                });

{
  my $loader;
  
  my $runfolder_path = qq{t/data/nfs/sf44/IL6/outgoing/100125_IL6_4308};

  lives_ok { $loader = npg_qc::api::loader::Gerald_Summary->new({
                            runfolder_path => $runfolder_path,
                            util => $util,
  }); } q{loader object creation ok};

  isa_ok($loader, q{npg_qc::api::loader::Gerald_Summary}, q{$loader});

  is($loader->xml_file(), q{t/data/nfs/sf44/IL6/outgoing/100125_IL6_4308/Data/Intensities/Bustard1.6.0a14_29-01-2010_RTA/GERALD_29-01-2010_RTA/Summary.xml}, 'correct xml file name');
 
  lives_ok { $loader->run(); } q{4308 runs ok};
}

{
  my $loader;
  
  my $runfolder_path = qq{t/data/nfs/sf44/IL3/analysis/100117_IL3_4270};
  
  lives_ok { $loader = npg_qc::api::loader::Gerald_Summary->new({
                            runfolder_path => $runfolder_path,
                            util => $util,
  }); } q{loader object creation ok};

  isa_ok($loader, q{npg_qc::api::loader::Gerald_Summary}, q{$loader});

  is($loader->xml_file(), q{t/data/nfs/sf44/IL3/analysis/100117_IL3_4270/Data/Intensities/Bustard1.6.0a14_24-01-2010_RTA/GERALD_24-01-2010_RTA/Summary.xml}, 'correct xml file name');
 
  lives_ok { $loader->run(); } q{4270 runs ok};
}

1;
