use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';
use Test::Exception;
use lib qw{t};
use SangerPaths qw(core);
use t::api::util;

BEGIN {
  use_ok(q{npg_qc::api::loader::Matrix});
}

my $util = t::api::util->new({
                dbh => 'dummy',
                fixtures => 1,
                useragent => undef,
                });

{
  #test for paird run
  my $loader;
  
  my $runfolder_path = qq{t/data/nfs/sf44/IL6/outgoing/100125_IL6_4308};

  lives_ok { $loader = npg_qc::api::loader::Matrix->new({
                            runfolder_path => $runfolder_path,
                            util => $util,
  }); } q{loader object creation ok};

  isa_ok($loader, q{npg_qc::api::loader::Matrix}, q{$loader});

  lives_ok { $loader->run(); } q{4308 matrix runs ok};
}

{
  #test for paird run
  my $loader;

  my $archive_path = q{t/data/nfs/sf44/ILorHSany_sf25/incoming/100925_HS8_05330_B_205NNABXX/Data/Intensities/PB_basecalls_20101014-161249/no_cal/archive};

  lives_ok { $loader = npg_qc::api::loader::Matrix->new({
                            archive_path => $archive_path,
                            util => $util,
  }); } q{loader object creation ok};

  isa_ok($loader, q{npg_qc::api::loader::Matrix}, q{$loader});
  
  is($loader->_second_cycle_number_by_read(1), 2, 'correct second cycle number for read 1');
  is($loader->_second_cycle_number_by_read(2), 102, 'correct second cycle number for read 2');

  lives_ok { $loader->run(); } q{5330 matrix runs ok};
}

1;
