use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';
use Test::Exception;
use lib qw{t};
use SangerPaths qw(core);
use t::api::util;
use t::useragent;

BEGIN {
  use_ok(q{npg_qc::api::monitoring::Runinfo});
}

my $ua_mock = {};
my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
my $npg_api_util = npg::api::util->new({useragent => $ua});
my $util = t::api::util->new({
                npg_api_util => $npg_api_util,
                fixtures => 1,
                useragent => $ua,
                });

{

  my $runinfo_monitor;

  lives_ok { $runinfo_monitor = npg_qc::api::monitoring::Runinfo->new({
                            pattern_prefix  => 't/data/nfs/sf44/IL*/',
                            util => $util,
  }); } q{loader object creation ok};

  isa_ok($runinfo_monitor, q{npg_qc::api::monitoring::Runinfo}, q{$runinfo_monitor});
  
  is(scalar keys %{$runinfo_monitor->runlist_db()}, 0, 'correct number of runs in db');
  is(scalar @{$runinfo_monitor->runfolder_list_todo()}, 2, 'correct number of runs to do');

  lives_ok { $runinfo_monitor->run(); } q{runlog loading ok};
}

1;
