use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';
use Test::Exception;
use lib qw{t};
use SangerPaths qw(core);
use t::api::util;
use t::useragent;

BEGIN {
  use_ok(q{npg_qc::api::monitoring::Runlog});
}

my $ua_mock = {
     q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query} => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
     q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//run_tile/;list_movez_id_run_xml} => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/movez_id_run.xml},
     q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run/4308}         => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run/4308.xml},
  };
  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 1, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/'});

{

  my $runlog_monitor;

  lives_ok { $runlog_monitor = npg_qc::api::monitoring::Runlog->new({
                            pattern_prefix  => 't/data/nfs/sf44/IL*/',
                            util => $util,
  }); } q{loader object creation ok};

  isa_ok($runlog_monitor, q{npg_qc::api::monitoring::Runlog}, q{$runlog_monitor});
  
  is(scalar keys %{$runlog_monitor->runlist_db()}, 7, 'correct number of runs in db');
  is(scalar @{$runlog_monitor->runfolder_list_todo()}, 2, 'correct number of runs to do');
  shift @{$runlog_monitor->runfolder_list_todo()};
  lives_ok { $runlog_monitor->run(); } q{runlog loading ok};
}

1;
