#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-07-23
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 60-autoqc-db-DB.t 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/t/60-autoqc-db-DB.t $
#

use strict;
use warnings;
use Test::More tests => 60;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use npg::api::util;
use t::useragent;
use Carp;

use npg_qc::autoqc::qc_store::options qw/$ALL $LANES $PLEXES/;

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mx; $r; };

#:

$ENV{dev} = 'test';

use_ok('npg_qc::autoqc::db::DB');

my $util = t::api::util->new({fixtures => 1, dbh => undef, useragent => undef});
my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db ={ dsn=>"DBI:mysql:database=$dbname;host=$dbhost;port=$dbport", username=>$user, password => $password };

npg_qc::autoqc::db::Schema->load_schema($db);

my $class_name = npg_qc::autoqc::db::DB->_get_package_from_json(q{{"check_name":"insert_size",
 "id_run":3510,
 "__CLASS__":"npg_qc::autoqc::results::insert_size-5467",
 "filename":["3510_1_1.fastq","3510_1_2.fastq"],
 "expected_mean":400,
 "position":1}});
is($class_name, q{npg_qc::autoqc::results::insert_size}, "correct class name from json string");

my $insert_size1; 
eval{
  $insert_size1 = npg_qc::autoqc::db::DB->save_from_json(q{{"id_run": 3510,
  "position":3,
  "__CLASS__":"npg_qc::autoqc::results::insert_size-5467",
  "path": "/tmp",
  "filenames": ["3510_1_1.fastq","3510_1_2.fastq"],
  "bins":[1, 3, 5],
  "bin_width":5,
  "min_isize":400,
  "expected_mean":10000,
  "mean":1000,
  "std":50,
  "sample_size": 22000,
  "num_well_aligned_reads": 20000,
  "quartile1": 3000,
  "median":3000,
  "quartile3": 40000,
  "pass":1
  }});
  1;
};
is($EVAL_ERROR, q{}, "no croak when save insert_size from a json string");
is($insert_size1->id_insert_size(), 2, "insert size json string saved with key 2");

my $qx_yield1;
eval{
  $qx_yield1 = npg_qc::autoqc::db::DB->save_from_jsonfile("t/data/autoqc/3510_4.qx_yield.json");
  1;
};
is($EVAL_ERROR, q{}, "no croak when save qx_yield from a json file");
is($qx_yield1->id_qx_yield(), 12, 'data saved from the json file and the primary key is 12');

my $qx_yield_lane_count = npg_qc::autoqc::db::DB->lane_count_per_run_db('qX_yield', 3510);
is($qx_yield_lane_count , 1, 'correct lane count in db for run 3510 qx_yield');

my $insert_size_lane_count = npg_qc::autoqc::db::DB->lane_count_per_run_db('insert_size', 3510);
is($insert_size_lane_count , 1, 'correct lane count in db for run 3510 insert_size');

is(scalar @{npg_qc::autoqc::db::DB->list_per_run(3510, 'qX_yield')}, 1, 'correct number of qX_yield for run 3510');
is(scalar @{npg_qc::autoqc::db::DB->list_per_run(3510, 'insert_size')}, 1, 'correct number of insert size for run 3510');

is(scalar @{npg_qc::autoqc::db::checks::qX_yield->list_per_run(3510)}, 1, 'correct number of qX_yield for run 3510');
is(scalar @{npg_qc::autoqc::db::checks::insert_size->list_per_run(3510)}, 1, 'correct number of insert size for run 3510');

my $collection = npg_qc::autoqc::db::DB->autoqc_data_by_run_db(3500);
is($collection->size(), 8, 'correct number of autoqc objects return');
$collection->sort_collection(q[check_name]);
my $results =  $collection->results;
isa_ok($results->[3], 'npg_qc::autoqc::db::checks::qX_yield');
isa_ok($results->[2], 'npg_qc::autoqc::db::checks::insert_size');
isa_ok($results->[4], 'npg_qc::autoqc::db::checks::sequence_error');
isa_ok($results->[1], 'npg_qc::autoqc::db::checks::gc_fraction');
isa_ok($results->[0], 'npg_qc::autoqc::db::checks::adapter');
isa_ok($results->[6], 'npg_qc::autoqc::db::checks::split_stats');
isa_ok($results->[7], 'npg_qc::autoqc::db::checks::tag_decode_stats');

is($collection->results->[6]->alignment_coverage1()->{'x'}, 0.1234, 'correct coverage value for fastq 1 seq x run 3500');
is($collection->results->[6]->alignment_coverage2()->{'y'}, 0.1237, 'correct coverage value for fastq 2 seq y run 3500');

my ($id_run, $position) 
 = npg_qc::autoqc::db::DB->_get_id_run_position_from_filename('/tmp/3124_8_1-3124_8_2_human_split_stats.json');
is($id_run, 3124, 'correct id_run from filename');
is($position, 8, 'correct position from filename');

{
  my $collection = npg_qc::autoqc::db::DB->autoqc_data_by_run_db(3600);
  is($collection->size(), 4, 'autoqc objects for all lanes');

  $collection = npg_qc::autoqc::db::DB->autoqc_data_by_run_db(3600, []);
  is($collection->size(), 4, 'autoqc objects for all lanes');

  $collection = npg_qc::autoqc::db::DB->autoqc_data_by_run_db(3600, undef);
  is($collection->size(), 4, 'autoqc objects for all lanes');

  $collection = npg_qc::autoqc::db::DB->autoqc_data_by_run_db(3600, [1,2]);
  is($collection->size(), 2, 'autoqc objects for two lanes');
  my $results =  $collection->results;
  is ($results->[0]->position, 1, 'result for position 1 is first');
  is ($results->[1]->position, 2, 'result for position 2 is second');
}


{
  my $collection = npg_qc::autoqc::db::DB->autoqc_data_by_run_db(3600, undef, $LANES);
  is($collection->size(), 4, 'autoqc objects for all lanes');

  $collection = npg_qc::autoqc::db::DB->autoqc_data_by_run_db(3600, [], $ALL);
  is($collection->size(), 10, 'autoqc objects for all lanes and plexes');

  $collection = npg_qc::autoqc::db::DB->autoqc_data_by_run_db(3600, undef, $PLEXES);
  is($collection->size(), 6, 'autoqc objects for plexes only');

  $collection = npg_qc::autoqc::db::DB->autoqc_data_by_run_db(3600, [1,2], $PLEXES);
  is($collection->size(), 0, 'autoqc objects for plexes for lanes with no plexes');

  $collection = npg_qc::autoqc::db::DB->autoqc_data_by_run_db(3600, [3,4], $PLEXES);
  is($collection->size(), 6, 'autoqc objects for plexes for two lanes');

  $collection = npg_qc::autoqc::db::DB->autoqc_data_by_run_db(3600, [3,4], $ALL);
  is($collection->size(), 8, 'all autoqc objects for two lanes');

  $collection = npg_qc::autoqc::db::DB->autoqc_data_by_run_db(3600, [3,4], $LANES);
  is($collection->size(), 2, 'autoqc objects for two lanes when plexes are also available');
}

{
  is( npg_qc::autoqc::db::DB->_autoqc_subtype_method_name(q{split_stats}), q{ref_name} , 'correct subtype for split_stats');
  is( npg_qc::autoqc::db::DB->_autoqc_subtype_method_name(q{bam_flagstats}), q{human_split} , 'correct subtype for bam_flagstats');
  is( npg_qc::autoqc::db::DB->_autoqc_subtype_method_name(q{sequence_error}), q{sequence_type} , 'correct subtype for sequence_error');
  ok( ! npg_qc::autoqc::db::DB->_autoqc_subtype_method_name(q{insert_size}), 'no subtype for insert_size');
}

{
  my $count_archived_list = npg_qc::autoqc::db::DB->count_archived_list(3600);
  is( $count_archived_list->{list_by_lane}->{qX_yield}->{3} , 1, 'correct number for run 3600 lane 3 qX_yield');
  is( $count_archived_list->{list_by_lane_tag}->{qX_yield}->{4}->{4}, 1, 'correct number for run 3600 lane 4 tag 4 qX_yield');
}

{
  my $count_archived_list = npg_qc::autoqc::db::DB->count_archived_list(3500);
  is( $count_archived_list->{list_by_lane}->{sequence_error}->{1} , 1, 'correct number for run 3500 lane 1 sequence error');
  is( $count_archived_list->{list_by_lane_split}->{sequence_error}->{1}->{phix} , 1, 'correct number for run 3500 lane 1 phix sequence error');
  is( $count_archived_list->{list_by_lane_split}->{split_stats}->{1}->{human} , 1, 'correct number for run 3500 lane 1 human split stats');
}

{
    my $args_ref   = {
       lanes               => [1, 2],
       tag_list_by_lnae    => {},
       spiked_phix_lanes   => {},
       non_consented_lanes => {},
       check_name          => q{insert_size},
       list_by_lane        => {1=>1,},
    };
    ok(!npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 2 insert size missing');
    $args_ref->{list_by_lane}->{2} = 1;
    ok(npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 1 and 2 insert size ok');
}

{
    my $args_ref   = {
       lanes               => [1, 2],
       tag_list_by_lnae    => {},
       spiked_phix_lanes   => {},
       non_consented_lanes => {},
       check_name          => q{sequence_error},
       list_by_lane        => {1=>1,},
    };
    ok(!npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 2 sequence error missing');
    $args_ref->{list_by_lane}->{2} = 1;
    ok(npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 1 and 2 sequence error ok');
    $args_ref->{spiked_phix_lanes } = {1=>1};
    ok(!npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 1 phix sequence error missing');
    $args_ref->{list_by_lane_split}->{1} = { phix => 1};
    ok(npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 1 phix sequence error ok');
}

{
    my $args_ref   = {
       lanes               => [1, 2, 3],
       tag_list_by_lane    => {},
       spiked_phix_lanes   => {},
       non_consented_lanes => {},
       check_name          => q{bam_flagstats},
       list_by_lane        => {1=>1, 2 => 1, },
    };
    ok(!npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 3 bam flagstats missing');
    $args_ref->{list_by_lane}->{3} = 1;
    ok(npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 1, 2 and 3 bam flagstats ok');
    $args_ref->{non_consented_lanes} = { 2 => 1 };
    ok(!npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 2 human and nonhuman bam flagstats missing');
    $args_ref->{list_by_lane_split}->{2} = { human => 1 };
    ok(!npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 2 nonhuman bam flagstats missing');
    $args_ref->{list_by_lane_split}->{2} = { human => 1, nonhuman => 1};
    ok(npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 2 nonhuman and human bam flagstats missing');
}
{
    my $args_ref   = {
       lanes               => [5],
       tag_list_by_lane    => { 5 => [0,1], },
       spiked_phix_lanes   => {},
       non_consented_lanes => {},
       check_name          => q{insert_size},
       list_by_lane        => { 5=>1,},
    };
    ok(!npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 5 tag 0, 1 insert size missing');
    $args_ref->{list_by_lane_tag}->{5} = {0 => 1, 1=>1};
    ok(npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 5 tag 0, 1 insert size ok');
    
    $args_ref->{check_name} = q{tag_decode_stats};
    $args_ref->{list_by_lane_tag}->{5} = {};
    ok(npg_qc::autoqc::db::DB->_one_check_data_completeness($args_ref), 'lane 5 tag_decode_stats ok');
}

1;
