#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2009-09-04
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 60-autoqc-db-Loader.t 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/t/60-autoqc-db-Loader.t $
#

use strict;
use warnings;
use Test::More tests => 19;
use Test::Exception;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw(core ensembl);
use Carp;
use t::api::util;

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mx; $r; };

$ENV{dev} = 'test';

use_ok('npg_qc::autoqc::db::Loader');
{
  my $loader = npg_qc::autoqc::db::Loader->new(path => ['t/data/autoqc/rendered/json_paired_run'], id_run => 3565);
  isa_ok($loader, 'npg_qc::autoqc::db::Loader', 'isa_ok');
}

{
  throws_ok {npg_qc::autoqc::db::Loader->new(path => ['t/data/autoqc/rendered/json_paired_run']) } 
       qr/Attribute \(id_run\) is required/, 'error when id_run is not specified';
  throws_ok {npg_qc::autoqc::db::Loader->new(id_run => 67) } 
       qr/Attribute \(path\) is required/, 'error when path is not specified';
  throws_ok {npg_qc::autoqc::db::Loader->new(id_run => 67, path => [],) } 
       qr/Array of json file locations is empty/, 'error when path is given as an empty array';
  throws_ok {npg_qc::autoqc::db::Loader->new(id_run => 67, path => ['t/data/autoqc/rendered/json_paired_run', ''],) } 
       qr/Empty string given as one of the paths/, 'error when one of the paths is an empty string';
  throws_ok {npg_qc::autoqc::db::Loader->new(id_run => 67, path => ['t/data/autoqc/rendered/json_paired_run', 'non-existing'],) } 
       qr/Non-existing path non-existing as one of the paths/, 'error when one of the paths does not exist';
  throws_ok {npg_qc::autoqc::db::Loader->new(id_run => 67, path => ['t/data/autoqc/rendered/json_paired_run', 't/data/autoqc/rendered/json_paired_run/3565_1.insert_size.json'],) } 
       qr/One of the paths, t\/data\/autoqc\/rendered\/json_paired_run\/3565_1\.insert_size\.json, is not a directory/, 'error when one of the paths is not a directory';
}

{
  my $loader = npg_qc::autoqc::db::Loader->new(path => ['t/data/autoqc/090908_IL11_3715'], id_run => 3715);
  is(scalar @{$loader->json_files}, 0, 'zero json files found');
  throws_ok { $loader->load() } qr/No json files found in given directories/,  'error when no json files are found';
}

my $util = t::api::util->new({fixtures => 1, dbh => undef, useragent => undef});

my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db ={ dsn=>"DBI:mysql:database=$dbname;host=$dbhost;port=$dbport", username=>$user, password => $password };

npg_qc::autoqc::db::Schema->load_schema($db);

{
  my $loader = npg_qc::autoqc::db::Loader->new(path => ['t/data/autoqc/rendered/json_paired_run'], id_run => 3565);
   ok($loader->num_db_records() == 0, 'no autoqc data in the db for run 3565 to start with');
  is(scalar @{$loader->json_files()}, 16, "correct number of json files got");

  my $count_json_files_loaded;
  lives_ok { $count_json_files_loaded = $loader->process_json_files() } 'no croak to load all json files for run 3565';
  is($count_json_files_loaded, 16, '16 files loaded for run 3565');

  lives_ok { $loader->load() } "no croak when reload the data into the dabase";
  ok($loader->num_db_records() == 16, 'run 3565 auto qc data are complete in db');
}

{
  my $loader2 = npg_qc::autoqc::db::Loader->new(path => ['t/data/autoqc/090908_IL11_3715/qc/'], id_run => 3715);
  is(scalar @{$loader2->json_files()}, 4, "correct number of json files got");
  lives_ok { $loader2->load() } "no croak when loading adapter check json files";
}

{
  my $loader3 = npg_qc::autoqc::db::Loader->new(path => ['t/data/autoqc/090908_IL11_3715/qc', 't/data/autoqc/rendered/json_paired_run'], id_run => 3715);
  is(scalar @{$loader3->json_files()}, 20, "correct number of json files for loading from two directories");
}

1;
