#########
# Author:        jo3
# Maintainer:    $Author: ajb $
# Created:       2008-09-02
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 60-autoqc-db-checks-adapter.t 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/t/60-autoqc-db-checks-adapter.t $
#

use strict;
use warnings;
use Test::More tests => 10;
use Test::Exception;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw( trackingnpg core );
use t::api::util;
use npg_qc::autoqc::db::Schema;

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/msx; $r; };

local $ENV{dev} = 'test';

use_ok('npg_qc::autoqc::db::checks::adapter');


my $util = t::api::util->new( { fixtures  => 1,
                                dbh       => undef,
                                useragent => undef
                              }
);

my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db = { dsn      => "DBI:mysql:database=$dbname;host=$dbhost;port=$dbport",
           username => $user,
           password => $password };

npg_qc::autoqc::db::Schema->load_schema($db);

my $adapter = npg_qc::autoqc::db::checks::adapter->new( id_adapter => 1 );

my @attributes = $adapter->meta->get_all_attributes();

#foreach my $attr (@attributes) {
#    my $name = $attr->name();
#    print $name, "\t" || carp;
#    print $attr->type_constraint(), "\t" || carp;
#    print $adapter->$name(), "\n" || carp;
#}
is( scalar @attributes,   24, 'Correct number of attributes' );
is( $adapter->id_run(),   3500, 'Correct id run' );
is( $adapter->position(),    1, 'Correct position' );
isa_ok( $adapter->forward_blat_hash(), 'HASH', 'Adapter hits is a hash' );

my $adapter2;
lives_ok {
    $adapter2 =
        npg_qc::autoqc::db::checks::adapter->new( id_run   => 3500,
                                                  position =>    1, 
                                                  tag_index => -1,)
}
'No croak when query database using id_run 3500, position 1';

is( $adapter2->id_adapter(), 1, 'Correct table index' );
is ( $adapter2->tag_index, undef, 'Tag index undefined' );

my $adapter3 = npg_qc::autoqc::db::checks::adapter->save_from_json(
    q{ { "id_run": 3510,
         "position":3,
         "__CLASS__":"npg_qc::autoqc::results::adapter-5467",
         "path": "/tmp",
         "forward_fasta_read_count": 10050000,
         "forward_contaminated_read_count": 53,
         "forward_blat_hash": {"adaptor1": 1, "DpnII-Gex-PCR-Primer1": 53},
         "reverse_fasta_read_count": 1430000,
         "reverse_contaminated_read_count": 26,
         "reverse_blat_hash": {"DpnII-Gex-Adapter2-1": 0},
         "pass": 1 }
     }
);


is( $adapter3->id_adapter(), 2,
    'Data saved from the JSON string. Primary key is 2' );


my $adapter4 = npg_qc::autoqc::db::checks::adapter->save_from_json(
q[
{"info":{"Check":"npg_qc::autoqc::checks::adapter","Check_version":"9254"},"forward_contaminated_read_count":5607114,"forward_blat_hash":{"PE-sequencingPrimer1":1140,"DpnII-Gex-Adapter2-1":0,"NlaIII-Gex-PCR-Primer1":0,"smallRNA-3'adapter":0,"DpnII-Gex-Adapter1-1":0,"PE-sequencingPrimer2":5497708,"DpnII-Gex-SequencingPrimer":0,"adaptor3":3611,"genomicDNA=primer2":0,"smallRNA-5'adapter":0,"PE-adapters1-2":1140,"NlaIII-Gex-Adapter2-1":0,"genomicDNA-primer1":3611,"NlaIII-Gex-Adapter1-1":0,"DpnII-Gex-PCR-Primer2":0,"smallRNA-PCS-primer1":0,"smallRNA-RT-primer":0,"NlaIII-Gex-Adapter2-2":0,"adaptor4":5571599,"NlaIII-Gex-Adapter1-2":0,"NlaIII-Gex-PCR-Primer2":0,"adaptor2":0,"DpnII-Gex-Adapter1-2":0,"genomicDNA-adapter2":1140,"genomicDNA-sequencingPrimer":1140,"PE-PCR-Primers1-1":3611,"PE-PCR-Primers1-2":5571599,"PE-adapters1-1":4842336,"smallRNA-sequencingPrimer":0,"NlaIII-Gex-SequencingPrimer":0,"adaptor1":4979241,"DpnII-Gex-PCR-Primer1":0,"smallRNA-PCS-primer2":0,"genomicDNA-adapter1":0,"DpnII-Gex-Adapter2-2":0},"reverse_start_counts":{"33":18,"32":28,"21":63,"7":26,"26":17,"17":46,"2":61,"1":8411,"18":49,"30":39,"16":44,"44":182,"27":63,"25":54,"28":31,"40":38,"20":108,"14":66,"24":56,"10":210,"31":76,"35":48,"11":43,"42":123,"22":71,"46":25,"13":43,"23":86,"29":149,"6":15,"39":65,"36":59,"3":10,"9":16,"41":416,"12":46,"47":72,"15":34,"38":62,"8":55,"4":45,"34":34,"45":109,"37":29,"19":55,"43":88,"5":20},"position":"7","forward_fasta_read_count":32222880,"path":"/staging/IL19/analysis/101020_IL19_05408/Latest_Summary/archive","id_run":"5408","forward_read_filename":"5408_7_1.fastq","__CLASS__":"npg_qc::autoqc::results::adapter-8942","reverse_blat_hash":{"PE-sequencingPrimer1":2829,"DpnII-Gex-Adapter2-1":0,"NlaIII-Gex-PCR-Primer1":0,"smallRNA-3'adapter":0,"DpnII-Gex-Adapter1-1":0,"PE-sequencingPrimer2":1,"DpnII-Gex-SequencingPrimer":0,"adaptor3":11438,"genomicDNA=primer2":0,"smallRNA-5'adapter":0,"PE-adapters1-2":2829,"NlaIII-Gex-Adapter2-1":0,"genomicDNA-primer1":11438,"NlaIII-Gex-Adapter1-1":0,"DpnII-Gex-PCR-Primer2":0,"smallRNA-PCS-primer1":0,"smallRNA-RT-primer":0,"NlaIII-Gex-Adapter2-2":0,"adaptor4":4,"NlaIII-Gex-Adapter1-2":0,"NlaIII-Gex-PCR-Primer2":0,"adaptor2":0,"DpnII-Gex-Adapter1-2":0,"genomicDNA-adapter2":2829,"genomicDNA-sequencingPrimer":2829,"PE-PCR-Primers1-1":11438,"PE-PCR-Primers1-2":4,"PE-adapters1-1":0,"smallRNA-sequencingPrimer":0,"NlaIII-Gex-SequencingPrimer":0,"adaptor1":0,"DpnII-Gex-PCR-Primer1":0,"smallRNA-PCS-primer2":0,"genomicDNA-adapter1":0,"DpnII-Gex-Adapter2-2":0},"reverse_contaminated_read_count":11504,"forward_start_counts":{"33":160,"32":136,"21":230,"7":1006,"26":187,"2":26789,"17":154,"1":5544809,"18":167,"30":162,"16":191,"44":192,"27":184,"25":159,"28":125,"40":78,"20":207,"14":584,"24":402,"10":2403,"31":207,"35":119,"11":487,"42":47,"22":283,"46":48,"23":296,"13":365,"29":180,"6":813,"39":109,"3":9503,"36":96,"9":1444,"41":87,"12":391,"15":156,"47":77,"38":113,"8":1290,"4":6446,"34":180,"45":100,"37":116,"43":120,"19":237,"5":5479},"reverse_read_filename":"5408_7_2.fastq","reverse_fasta_read_count":32222880}
]
);

is( $adapter4->id_adapter(), 3,
    'Data saved from the JSON string. Primary key is 3' );
1;
