#########
# Author:        jo3
# Maintainer:    $Author: ajb $
# Created:       2008-09-02
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 60-autoqc-db-checks-gc_bias.t 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/t/60-autoqc-db-checks-gc_bias.t $
#

use strict;
use warnings;
use Test::More 'no_plan';
use Test::Deep;
use Test::Exception;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw( trackingnpg core );
use t::api::util;
use Carp;

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/msx; $r; };

local $ENV{dev} = 'test';

use_ok('npg_qc::autoqc::db::checks::gc_bias');


my $util = t::api::util->new( { fixtures  => 1,
                                dbh       => undef,
                                useragent => undef, }
);

my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db = { dsn      => "DBI:mysql:database=$dbname;host=$dbhost;port=$dbport",
           username => $user,
           password => $password, };

npg_qc::autoqc::db::Schema->load_schema($db);

my $gc_bias    = npg_qc::autoqc::db::checks::gc_bias->new( id_gc_bias => 1 );
my @attributes = $gc_bias->meta->get_all_attributes();

# Uncomment this loop to trouble-shoot the attributes.
#foreach my $attr (@attributes) {
#    my $name = $attr->name();
#    # next if $name !~ m/raw$/msx;
#    my $tc = $attr->type_constraint() || 'no_type_constraint';
#    printf "%s\t%s\t%s\n", $name, $tc, $gc_bias->$name() or carp;
#}

is( scalar @attributes, 29, 'correct number of attributes' );
is( $gc_bias->id_run(),       3300,       'correct id run' );
is( $gc_bias->position(),        7,       'correct position' );
is( $gc_bias->bin_count(),      10,       'correct bin count' );
is( $gc_bias->window_count(), 2600,       'correct window count' );
is( $gc_bias->window_size(),   100,       'correct window size' );
is( $gc_bias->max_y(),         620.5,     'correct max_y' );
is( $gc_bias->pass(),            1,       'Pass value present' );
is( $gc_bias->path(),     '/tmp',         'correct path' );
is( $gc_bias->comments(), 'gc_bias test', 'correct comments' );
is( $gc_bias->cached_plot(),
    'iVBORw0KGgoAAAANSUhEUgAAAlgAAAJYCAMAAAAASUVORK5CYII=\n',
    'correct base64 plot' );

isa_ok( $gc_bias->plot_x(),               'ARRAY', 'X ordinates in array' );
isa_ok( $gc_bias->plot_y(),               'ARRAY', 'Y ordinates in array' );
isa_ok( $gc_bias->gc_lines(),             'ARRAY', 'GC_lines in array' );
isa_ok( $gc_bias->info(),                 'HASH',  'Info in hash' );
isa_ok( $gc_bias->ideal_lower_quantile(), 'ARRAY',
        'Ideal lower quantiles in array' );
isa_ok( $gc_bias->ideal_upper_quantile(), 'ARRAY',
        'Ideal upper quantiles in array' );
isa_ok( $gc_bias->actual_quantile_x(),    'ARRAY',
        'Actual x quantiles in array' );
isa_ok( $gc_bias->actual_quantile_y(),    'ARRAY',
        'Actual y quantiles in array' );


cmp_deeply( $gc_bias->plot_x(), [2,4,6,8,10],  'correct plot_x' );
cmp_deeply( $gc_bias->plot_y(), [4,8,12,16,20],'correct plot_y' );
cmp_deeply( $gc_bias->ideal_lower_quantile(),
            [3,6,9,12,15],   'correct ideal lower quantiles' );
cmp_deeply( $gc_bias->ideal_upper_quantile(),
            [5,10,15,20,25], 'correct ideal upper quantiles' );
cmp_deeply( $gc_bias->actual_quantile_x(),
            [2,4,6,8,10,10,8,6,4,2], 'correct actual x quantiles' );
cmp_deeply( $gc_bias->actual_quantile_y(),
            [5,9,13,17,21,19,15,11,7,3], 'correct actual y quantiles' );
cmp_deeply( $gc_bias->gc_lines(), [20,50,80], 'correct gc_lines' );
cmp_deeply( $gc_bias->info(),
            { 'Check'         => 'npg_qc::autoqc::checks::gc_bias',
              'Check_version' => '8853' },
            'correct info' );

my $gcb2;
lives_ok { $gcb2 =
               npg_qc::autoqc::db::checks::gc_bias->new( id_run    => 3300,
                                                         position  =>    7,
                                                         tag_index =>   -1, )
}
'Successfully query database with run, lane and tag index';

is( $gcb2->id_gc_bias(), 1, 'Correct table index' );
is( $gcb2->tag_index(), undef, 'Tag index undefined' );

my $gcb3 = npg_qc::autoqc::db::checks::gc_bias->save_from_json(
               q{ { "id_run": 3301,
                    "position": 8,
                    "path": "/tmp",
                    "__CLASS__": "npg_qc::autoqc::results::gc_bias-5467",
                    "pass": 1,
                    "comments": "This, too, is a test",
                    "bin_count": 7,
                    "window_count": 200,
                    "window_size": 1000,
                    "max_y": 6208.5,
                    "plot_x": [2,5,6,8,11],
                    "ideal_lower_quantile": [1,6,9,13,15],
                    "ideal_upper_quantile": [5,9,18,20,25],
                    "actual_quantile_x": [2,4,6,7,10,10,8,7,4,2],
                    "actual_quantile_y": [4,9,13,17,21,19,16,11,7,3],
                    "plot_y": [19,18,15,11,22],
                    "gc_lines": [25,60,85],
                    "cached_plot": "iVBORw0KGhEUgAAJYCAMAAAAASUVORK5CYII=\n" }
                }
);

is( $gcb3->id_gc_bias(), 2, 'Successfully saved from JSON string' );

1;
