#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: gq1 $
# Created:       08 February 2010
# Last Modified: $Date: 2009-12-14 15:21:01 +0000 (Mon, 14 Dec 2009) $
# Id:            $Id: 60-autoqc-db-checks-qX_yield.t 7671 2009-12-14 15:21:01Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/branches/prerelease-24.0/t/60-autoqc-db-checks-qX_yield.t $
#

use strict;
use warnings;
use Test::More tests => 18;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use t::useragent;
use Carp;

our $VERSION = do { my ($r) = q$Revision: 7671 $ =~ /(\d+)/mx; $r; };

$ENV{dev} = 'test';

use_ok('npg_qc::autoqc::db::checks::gc_fraction');

my $util = t::api::util->new({fixtures => 1, dbh => undef, useragent => undef});

my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db ={ dsn=>"DBI:mysql:database=$dbname;host=$dbhost;port=$dbport", username=>$user, password => $password };

npg_qc::autoqc::db::Schema->load_schema($db);

{
  my $qobj = npg_qc::autoqc::db::checks::gc_fraction->new(id_gc_fraction => 1);

  my @attributes = $qobj->meta->get_all_attributes();

#  foreach my $attr (@attributes){
#    my $name = $attr->name();
#    print $name, "\t";
#    print $attr->type_constraint, "\t";
#    print $qobj->$name(), "\n";
#  }


  is(scalar @attributes, 17, 'correct number of attributes');
  is($qobj->id_run(), 3500, 'correct id run');
  is($qobj->position, 1, 'correct position');

  is($qobj->class_name(), 'gc_fraction', 'correct name');
  is($qobj->check_name(), 'gc fraction', 'correct check name');
}

{
  my $qobj;

  eval{
    $qobj = npg_qc::autoqc::db::checks::gc_fraction->new({id_run => 3500, position =>1, tag_index => -1,});
    1;
  };
  is($EVAL_ERROR, q{}, 'no croak when query database using id_run 3500 position 1');
  is($qobj->id_gc_fraction(), 1, 'correct id for gc_fraction of run 3500 position 1');
}

{
  my $qobj = npg_qc::autoqc::db::checks::gc_fraction->save_from_json(q{{"id_run":3510,
"position": 3,
"path": "/tmp",
"filename1":"3510_4_1.fastqcheck", 
"filename2": "3510_4_2.fastqcheck",
"__CLASS__": "npg_qc::autoqc::results::gc_fraction-6666",
"threshold_quality":20,
"threshold_yield1":40000,
"threshold_yield2":40001,
"yield1":5000,
"yield2":50001,
"pass":1
}});
  is($qobj->id_gc_fraction(), 2, 'data saved from the json string and the primary key is 2');

}

{
  my $qobj4 = npg_qc::autoqc::db::checks::gc_fraction->save_from_jsonfile("t/data/autoqc/3510_4.gc_fraction.json");
  is($qobj4->id_gc_fraction(), 3, 'data saved from the json file and the primary key is 3');


  my $qobjs = $qobj4->gc_fractions_per_run();
  my $qobj5 = $qobjs->[0];

  is(scalar @{$qobjs}, 2, '2 gc_fraction for run 3510 in database');
  isa_ok($qobj5, 'npg_qc::autoqc::db::checks::gc_fraction');
  is($qobj5->id_run(), 3510, "first qx_yield is for run 3510");
  is($qobj5->position(), 3, "first qx_yield is for lane 3");
}

{
  my $qobjs2 = npg_qc::autoqc::db::checks::gc_fraction->gc_fractions_per_run(3500);
  my $qobj6 = $qobjs2->[0];

  is(scalar @{$qobjs2}, 1, '1 gc_fraction for run 3500 in database');
  isa_ok($qobj6, 'npg_qc::autoqc::db::checks::gc_fraction');
  is($qobj6->id_run(), 3500, "first gc_fraction is for run 3500");
  is($qobj6->position(), 1, "first gc_fraction is for lane 1");
}

1;
