#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-09-02
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 60-autoqc-db-checks-insert_size.t 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/t/60-autoqc-db-checks-insert_size.t $
#

use strict;
use warnings;
use Test::More tests => 19;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use Carp;
use npg_qc::autoqc::db::Schema;

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mx; $r; };

$ENV{dev} = 'test';

my $util = t::api::util->new({fixtures => 1, dbh => undef, useragent => undef});

my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db ={ dsn=>"DBI:mysql:database=$dbname;host=$dbhost;port=$dbport", username=>$user, password => $password };

npg_qc::autoqc::db::Schema->load_schema($db);

use_ok('npg_qc::autoqc::db::checks::insert_size');

my $insert_size = npg_qc::autoqc::db::checks::insert_size->new(id_insert_size => 1);

my @attributes = $insert_size->meta->get_all_attributes();

#foreach my $attr (@attributes){
#  my $name = $attr->name();
#  print $name, "\t";
#  print $attr->type_constraint, "\t";
#  print $insert_size->$name(), "\n";
#}

is(scalar @attributes, 29, 'correct number of attributes');
is($insert_size->id_run, 3500, 'correct id run');
is($insert_size->position, 1, 'correct position');
is($insert_size->filenames->[1], '3500_1_2_fastq', 'correct second filename');
isa_ok($insert_size->bins, 'ARRAY', 'isize_xbins is array');

my $insert_size2;
eval{
  $insert_size2 = npg_qc::autoqc::db::checks::insert_size->new(id_run => 3500, position =>1, tag_index=>-1,);
  1;
};
is($EVAL_ERROR, q{}, 'no croak when query database using id_run 3500 position 1');
is ( $insert_size2->id_insert_size(), 1, 'correct id' );

my $insert_size3 = npg_qc::autoqc::db::checks::insert_size->save_from_json(q{
 {"id_run": 3510,
  "position":3,
  "__CLASS__":"npg_qc::autoqc::results::insert_size-5467",
  "path": "/tmp",
  "filenames": ["3510_1_1.fastq","3510_1_2.fastq"],
  "bins":[1, 3, 5],
  "bin_width":5,
  "min_iszie": 400,
  "expected_mean":10000,
  "mean":1000,
  "std":50,
  "sample_size": 22000,
  "num_well_aligned_reads": 20000,
  "quartile1": 3000,
  "median":3000,
  "quartile3": 40000,
  "expected_size": [340, 350],
  "reference": "/nfs/repository/d0031/references/Human/default/all/bwa/Homo_sapiens.NCBI36.48.dna.all.fa",
  "pass":1
  }});
is($insert_size3->id_insert_size(), 2, 'data saved from the json string and the primary key is 2');


my $insert_size4 = npg_qc::autoqc::db::checks::insert_size->save_from_jsonfile("t/data/autoqc/3510_1.insert_size.json");
is($insert_size4->id_insert_size(), 3, 'data saved from the json file and the primary key is 3');

is($insert_size4->image_url(), q{http://chart.apis.google.com/chart?chbh=5,1,1&chco=4D89F9&chd=t:1,3,5&chds=0,5&chs=650x300&cht=bvg&chtt=Insert+sizes:+run+3510,+position+1&chxr=0,400,410,1|1,0,5,1&chxt=x,y}, 'correct google url');


my $insert_sizes = $insert_size4->insert_sizes_per_run();
my $insert_size5 = $insert_sizes->[0];

is(scalar @{$insert_sizes}, 2, '2 insert_size for run 3500 in database');
isa_ok($insert_size5, 'npg_qc::autoqc::db::checks::insert_size');
is($insert_size5->id_run(), 3510, "first insert_size is for run 3510");
is($insert_size5->position(), 1, "first insert_size is for lane 1");

my $insert_sizes2 = npg_qc::autoqc::db::checks::insert_size->insert_sizes_per_run(3500);
my $insert_size6 = $insert_sizes2->[0];

is(scalar @{$insert_sizes2}, 1, '1 insert_size for run 3500 in database');
isa_ok($insert_size6, 'npg_qc::autoqc::db::checks::insert_size');
is($insert_size6->id_run(), 3500, "first insert_size is for run 3500");
is($insert_size6->position(), 1, "first insert_size is for lane 1");

1;
