#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-07-23
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 60-autoqc-db-checks-qX_yield.t 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/t/60-autoqc-db-checks-qX_yield.t $
#

use strict;
use warnings;
use Test::More tests => 19;
use Test::Exception;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use t::useragent;
use Carp;

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mx; $r; };

$ENV{dev} = 'test';

use_ok('npg_qc::autoqc::db::checks::qX_yield');

my $util = t::api::util->new({fixtures => 1, dbh => undef, useragent => undef});

my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db ={ dsn=>"DBI:mysql:database=$dbname;host=$dbhost;port=$dbport", username=>$user, password => $password };

npg_qc::autoqc::db::Schema->load_schema($db);

{
  my $qx_yield = npg_qc::autoqc::db::checks::qX_yield->new(id_qx_yield => 1);

  my @attributes = $qx_yield->meta->get_all_attributes();

#  foreach my $attr (@attributes){
#    my $name = $attr->name();
#    print $name, "\t";
#    print $attr->type_constraint, "\t";
#    print $qx_yield->$name(), "\n";
#  }


  is(scalar @attributes, 17, 'correct number of attributes');
  is($qx_yield->id_run(), 3500, 'correct id run');
  is($qx_yield->position, 1, 'correct position');

  is($qx_yield->class_name(), 'qX_yield', 'correct name');
  is($qx_yield->check_name(), 'qX yield', 'correct check name');
  is($qx_yield->pass_per_read(1), 1, 'first read pass');
}

{
  my $qx_yield;
  lives_ok {
    $qx_yield = npg_qc::autoqc::db::checks::qX_yield->new(id_run => 3500, position =>1, tag_index => -1,)
  } 'no croak when query database using id_run 3500 position 1';
  is($qx_yield->id_qx_yield(), 1, 'correct id for qx_yield of run 3500 position 1');
}

{
  my $qx_yield = npg_qc::autoqc::db::checks::qX_yield->save_from_json(q{{"id_run":3510,
"position": 3,
"path": "/tmp",
"filename1":"3510_4_1.fastqcheck", 
"filename2": "3510_4_2.fastqcheck",
"__CLASS__": "npg_qc::autoqc::results::qX_yield-6666",
"threshold_quality":20,
"threshold_yield1":40000,
"threshold_yield2":40001,
"yield1":5000,
"yield2":50001,
"pass":1
}});
  is($qx_yield->id_qx_yield(), 12, 'data saved from the json string and the primary key is 12');

}

{
  my $qx_yield4 = npg_qc::autoqc::db::checks::qX_yield->save_from_jsonfile("t/data/autoqc/3510_4.qx_yield.json");
  is($qx_yield4->id_qx_yield(), 13, 'data saved from the json file and the primary key is 13');


  my $qx_yields = $qx_yield4->qx_yields_per_run();
  my $qx_yield5 = $qx_yields->[0];

  is(scalar @{$qx_yields}, 2, '2 qx_yield for run 3510 in database');
  isa_ok($qx_yield5, 'npg_qc::autoqc::db::checks::qX_yield');
  is($qx_yield5->id_run(), 3510, "first qx_yield is for run 3510");
  is($qx_yield5->position(), 3, "first qx_yield is for lane 3");
}

{
  my $qx_yields2 = npg_qc::autoqc::db::checks::qX_yield->qx_yields_per_run(3500);
  my $qx_yield6 = $qx_yields2->[0];

  is(scalar @{$qx_yields2}, 1, '1 qx_yield for run 3500 in database');
  isa_ok($qx_yield6, 'npg_qc::autoqc::db::checks::qX_yield');
  is($qx_yield6->id_run(), 3500, "first qx_yield is for run 3500");
  is($qx_yield6->position(), 1, "first qx_yield is for lane 1");
}

1;
