#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-07-23
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 60-autoqc-db-checks-sequence_error.t 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/t/60-autoqc-db-checks-sequence_error.t $
#

use strict;
use warnings;
use Test::More tests => 22;
use Test::Exception;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use t::useragent;
use Carp;

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mx; $r; };

$ENV{dev} = 'test';

use_ok('npg_qc::autoqc::db::checks::sequence_error');

my $util = t::api::util->new({fixtures => 1, dbh => undef, useragent => undef});

my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db ={ dsn=>"DBI:mysql:database=$dbname;host=$dbhost;port=$dbport", username=>$user, password => $password };
npg_qc::autoqc::db::Schema->load_schema($db);


{
  my $error_check = npg_qc::autoqc::db::checks::sequence_error->new(id_sequence_error => 1);

  my @attributes = $error_check->meta->get_all_attributes();

#   foreach my $attr (@attributes){
#    my $name = $attr->name();
#    print $name, "\t";
#    print $attr->type_constraint, "\t";
#    print $error_check->$name(), "\n";
#  }

  is(scalar @attributes, 36, 'correct number of attributes');
  is($error_check->id_run(), 3500, 'correct id run');
  is($error_check->position, 1, 'correct position');

  is($error_check->class_name(), 'sequence_error', 'correct name');
  is($error_check->check_name(), 'sequence mismatch phix', 'correct check name');
  isa_ok($error_check->error_per_run()->[0], 'npg_qc::autoqc::db::checks::sequence_error');
}

{
  my $error_check;
  lives_ok {
    $error_check = npg_qc::autoqc::db::checks::sequence_error->new(id_run => 3500, position =>1, tag_index => -1, sequence_type => 'phix')
  } 'no croak when query database using id_run 3500 position 1';
  ok($error_check, 'object exists');
  is($error_check->id_sequence_error(), 1, 'correct id for error check of run 3500 position 1');
  is($error_check->tag_index(), undef, 'tag index undefined');
}

{
  my $error_check;
  lives_ok {
    $error_check = npg_qc::autoqc::db::checks::sequence_error->new(id_run => 3500, position =>1, tag_index => -1, sequence_type => 'default')
  } 'no croak when query database using id_run 3500 position 1';
  ok($error_check, 'object exists');
  is($error_check->id_sequence_error(), 2, 'correct id for error check of run 3500 position 1');
  is($error_check->tag_index(), undef, 'tag index undefined');
  is($error_check->sequence_type(), undef, "sequence type undefined");
}

{
  my $error_check = npg_qc::autoqc::db::checks::sequence_error->save_from_jsonfile("t/data/autoqc/3994_4.sequence_error.json");
  is($error_check->id_sequence_error(), 3, 'data saved from the json file and the primary key is 2');


  my $error_checks = $error_check->error_per_run();
  my $error_check2 = $error_checks->[0];

  is(scalar @{$error_checks }, 1, '1 error check for run 3994 in database');
  isa_ok($error_check2, 'npg_qc::autoqc::db::checks::sequence_error');
  is($error_check2->id_run(), 3994, "first error check is for run 3994");
  is($error_check2->position(), 4, "first error check is for lane 4");
  is($error_check2->sequence_type(), undef, "sequence type undefined");
}

1;
