#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: ajb $
# Created:       29 July 2009
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 60-autoqc-parse-alignment.t 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/t/60-autoqc-parse-alignment.t $
#

use strict;
use warnings;
use Test::More tests => 13;
use Test::Exception;
use Test::Deep;
use Cwd qw(cwd);
use File::Spec::Functions qw(catfile);

use npg_qc::autoqc::results::insert_size;

use_ok('npg_qc::autoqc::parse::alignment');

my $s1 = catfile(cwd, q[t/data/autoqc/alignment.sam]);
my $num_aligned = 10373;

my $result = npg_qc::autoqc::results::insert_size->new(id_run => 2, position => 1, path => q[t],);
$result->bin_width(6);
$result->min_isize(38);
$result->mean(349);
$result->std(46);
$result->quartile1(348);
$result->median(372);
$result->quartile3(387);
$result->bins([2,2,3,5,5,11,9,15,19,21,33,26,22,33,18,37,52,51,45,52,45,56,36,43,46,51,51,46,28,34,40,26,31,23,33,31,25,25,33,31,33,27,27,53,58,80,121,143,168,197,268,288,467,628,797,983,1035,1016,900,777,514,308,147,61,34,17,4,5,3,3,2,3,5,1,0,1,0,0,2,1,0,0,0,0,0,1]);
$result->num_well_aligned_reads($num_aligned);
$result->num_well_aligned_reads_opp_dir(undef);

{
  my $qc = npg_qc::autoqc::parse::alignment->new(files2parse => [$s1]);
  isa_ok($qc, 'npg_qc::autoqc::parse::alignment', 'is test with min constructor');
  throws_ok {npg_qc::autoqc::parse::alignment->new(files2parse => [q[non-existing]])}
        qr/Attribute \(files2parse\) does not pass the type constraint/, 'error when the first input file does not exist';
   throws_ok {npg_qc::autoqc::parse::alignment->new(files2parse => [$s1, q[non-existing]])}
        qr/Attribute \(files2parse\) does not pass the type constraint/, 'error when the second input file does not exist';
}

{
  my $qc = npg_qc::autoqc::parse::alignment->new(files2parse => [$s1]);
  is(scalar @{$qc->isizes_from_sam($s1)}, $num_aligned, 'num aligned reads returned');
  my $r = npg_qc::autoqc::results::insert_size->new(id_run => 2, position => 1, path => q[t],);
  is($qc->generate_insert_sizes($r), $num_aligned, 'num aligned reads returned');
  cmp_deeply ($r, $result, 'result object after parsing');
}

{
  my $qc = npg_qc::autoqc::parse::alignment->new(files2parse => [$s1, $s1]);
  my $r = npg_qc::autoqc::results::insert_size->new(id_run => 2, position => 1, path => q[t],);
  is($qc->generate_insert_sizes($r), $num_aligned, 'num aligned reads returned');
  $result->num_well_aligned_reads_opp_dir($num_aligned);
  cmp_deeply ($r, $result, 'result object after parsing');
}

SKIP: {
  skip('Bio::DB::Sam is not available', 4) 
            if !eval "require Bio::DB::Sam";
  my $b1 = $s1;
  $b1 =~ s/sam/bam/smx;
  my $qc = npg_qc::autoqc::parse::alignment->new(files2parse => [$b1]);
  is(scalar @{$qc->isizes_from_bam($b1)}, $num_aligned, 'num aligned reads returned - parse bam');
  is($qc->generate_insert_sizes(), $num_aligned, 'num aligned reads returned - parse bam');
  my $r = npg_qc::autoqc::results::insert_size->new(id_run => 2, position => 1, path => q[t],);
  is($qc->generate_insert_sizes($r), $num_aligned, 'num aligned reads returned - bam');
  cmp_deeply ($r, $result, 'result object after parsing - bam');
}

1;




