#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-06-23
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: 60-autoqc-results-bam_flagstats.t 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/t/60-autoqc-results-bam_flagstats.t $
#

use strict;
use warnings;
use Test::More tests => 15;
use English qw(-no_match_vars);
use Carp;
use File::Temp qw/ tempdir /;
use Perl6::Slurp;

our $VERSION = do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mx; $r; };

use_ok ('npg_qc::autoqc::results::bam_flagstats');

{
    my $tempdir = tempdir( CLEANUP => 1);
    my $r = npg_qc::autoqc::results::bam_flagstats->new({
                                       position => 5,
                                       id_run   => 4783,                                       
                                 });
    isa_ok ($r, 'npg_qc::autoqc::results::bam_flagstats');

    $r->parsing_metrics_file('t/data/autoqc/4783_5_metrics_optical.txt');

    open my $flagstats_fh, '<', 't/data/autoqc/4783_5_mk.flagstat';
    $r->parsing_flagstats($flagstats_fh);
    close $flagstats_fh;
  

    my $result_json;
    eval{
      $result_json = $r->freeze();
      $r->store(qq{$tempdir/4783_5_bam_flagstats.json});
      1;
    };
    is($EVAL_ERROR, q{}, 'no croak when save data into json');
    $result_json =~ s/"__CLASS__"\:"npg_qc::autoqc::results::bam_flagstats\-\d+"\,//;
    is($result_json, (slurp 't/data/autoqc/4783_5_bam_flagstats.json', {chomp=>1}), 'correct json output');
 
    is($r->total_reads(), 32737230 , 'total reads');
    is($r->total_mapped_reads(), '30992462', 'total mapped reads');
    is($r->percent_mapped_reads, 0.946703859795102, 'percent mapped reads');
    is($r->percent_duplicate_reads, 0.156023713120952, 'percent duplicate reads');
    is($r->percent_properly_paired ,0.897229484595978, 'percent properly paired');
    is($r->percent_singletons, 0.0292540938863795, 'percent singletons');
    is($r->check_name(), 'bam flagstats', 'correct check name');
    $r->human_split('human');
    is($r->check_name(), 'bam flagstats human', 'correct check name');
    
        
    open my $flagstats_fh2, '<', 't/data/autoqc/6440_1#0.bamflagstats';
    $r->parsing_flagstats($flagstats_fh2);
    close $flagstats_fh2; 
    is($r->total_reads(), 2978224 , 'total reads');
    is($r->proper_mapped_pair(),2765882, 'properly paired');
}

{
    my $r = npg_qc::autoqc::results::bam_flagstats->load('t/data/autoqc/4921_3_bam_flagstats.json');
    ok( !$r->total_reads(), 'total reads not available' ) ;
    
}
